unit AUTOREG; {Unit 9 of CONGRESS.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS, ROOTS, SPECTRAL, Dos, Crt;

  function SoftRand (var genSeed: longint): real;

  function WhiteNoise (var genSeed: longint): real;

  procedure ARMAgenerator (ns, nf, p, q: integer;
                  var alpha, mu: vector;
                  var x: longVector);

  procedure GenerateData (var ARMAGen: ARMARecord;
                  var realBOX: rBOX;
                  var dataVector: longVector);

  procedure GenerateBandLimitedData (var ARMAGen: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure AutoCovariances (Tcap, lags: integer;
                  var dataVector: longVector;
                  var aCovar: vector);

  procedure CircularACovars (Tcap, lags: integer;
                  var dataVector: longVector;
                  var cirCovar: longVector);

  procedure YuleWalker (p, q: integer;
                  var gamma: vector;
                  var alpha: vector;
                  var varEpsilon: real);

  procedure LevinsonDurbin (var gamma: vector;
                  var ARMAEst: ARMARecord;
                  pacv: vector);

  procedure MakeScreenBandSpec (var ARMAGen: ARMARecord;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

{Perhaps this one below should not refer to the Screen}
  procedure MakeScreenGenSpec (var ARMAGen: ARMARecord;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

  procedure MakeAREstimSpec (var ARMAEst: ARMARecord;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

  procedure MakePostBandSpec (var ARMAGen: ARMARecord;
                  var SCREENBOX: IBOX;
                  var specVec: longVector;
                  var specBOX: rBOX);

  procedure ARMACovariances (var ARMAGen: ARMARecord;
                  var gamma: vector);

  procedure ImposeZeros (var ARMAEst: ARMARecord);

  procedure GetPsivar (var ARMAEst: ARMARecord;
                  var aCovar: vector);

  procedure PoleZeroEstimates (var ARMAEst: ARMARecord);

  procedure ARMAparameters (var ARMAEst: ARMARecord;
                  var aCovar: vector);

  procedure ModelAnalysis (var ARMAGen, ARMAEst: ARMARecord;
                  var dataBOX: rBOX);


implementation

{_________________________________________________}

  function SoftRand (var genSeed: longint): real;

     {declare  <randStore: ARRAY [0..100] of longint;>}
     {and <genSeed: longint;> in the global environment.}

    const
      m = 714025;
      mVerse = 1.400512e-6;      {1.0/m}
      a = 1366;
      c = 150889;
    var
      j: integer;

  begin

    begin {initialise}

{if (genSeed = -1) then}
{genSeed := 99}
{fixed start point}
{else if (genSeed = -2) then}
{genSeed := seedStore;}
{repeat the last sequence}

      for j := 0 to 97 do
        begin {store some random numbers}
          genSeed := (a * genSeed + c) mod m;
          randStore[j] := genSeed
        end; {store some random numbers}
    end; {intiialise}

    j := 1 + (97 * randStore[0]) div m;
    randStore[0] := randStore[j];
    genSeed := (a * genSeed + c) mod m;
    randStore[j] := genSeed;
    SoftRand := randStore[0] * mVerse;

  end; {SoftRand}

{_________________________________________________}

  function WhiteNoise (var genSeed: longint): real;
    var
      u1, u2, s: real;    {<genSeed, seedStore: longint> are declared globally}
  begin{White_Noise}
    s := 2;
    while (s >= 1) or (s = 0) do
      begin
        u1 := 2.0 * SoftRand(genSeed) - 1.0;
        u2 := 2.0 * SoftRand(genSeed) - 1.0;
        s := Sqr(u1) + Sqr(u2);
      end;
    WhiteNoise := u1 * Sqrt(-2 * ln(s) / s);
  end;{WhiteNoise}

{_________________________________________________}

  procedure ARMAgenerator (ns, nf, p, q: integer;
                  var alpha, mu: vector;
                  var x: longVector);

    var
      i, j, r, t, n: integer;
      y, epsilon: vector;

  begin{ARMAgenerator}

    r := 10 * p;
    t := Max(p, q) + 1;
    n := ns + nf;

    for i := 0 to t do
      begin
        y[t - i] := 0;
        epsilon[t - i] := 0;
      end;

    for j := -r to n do
      begin {t}
        y[t] := 0.0;
        for i := p downto 1 do
          begin
            y[t] := y[t] - alpha[i] * y[t - i];
            y[t - i - 1] := y[t - i];
          end;
        for i := q downto 1 do
          begin
            y[t] := y[t] + mu[i] * epsilon[t - i];
            epsilon[t - i - 1] := epsilon[t - i];
          end;
        if j <= ns then
          epsilon[t] := WhiteNoise(genSeed)
        else
          epsilon[t] := 0;

        y[t] := y[t] + mu[0] * epsilon[t];
        epsilon[t - 1] := epsilon[t];
        y[t - 1] := y[t];

        if j >= 0 then
          x[j] := y[t];
      end; {j}

  end;{ARMAgenerator}

{_________________________________________________}

  procedure GenerateData (var ARMAGen: ARMARecord;
                  var realBOX: rBOX;
                  var dataVector: longVector);

    var
      i: integer;

  begin {Generate Data}
    with ARMAGen do
      begin

        ARMAgenerator(realBOX.NOPUNCT, 0, p, q, alpha, mu, dataVector);

        for i := 0 to realBOX.NOPUNCT - 1 do
          begin {i}
            if i = 0 then
              begin
                realBOX.yMax := dataVector[0];
                realBOX.yMin := dataVector[0];
              end;
            realBOX.yMax := Rmax(dataVector[i], realBOX.yMax);
            realBOX.yMin := Rmin(dataVector[i], realBOX.yMin);
          end; {i}

        realBOX.xMax := realBOX.NOPUNCT - 1;
        realBOX.xMin := 0.0;

        Variance(dataBOX, dataVector);

        ARMAGen.bandLimits := false;
        thereIsData := true;
        pseudoData := true;


      end;
  end; {GenerateData}

{________________________________________________}

  procedure GenerateBandLimitedData (var ARMAGen: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);
    var
      pi, fourierInc: real;
      t, Ncap, effectiveSize: integer;
      alpha, beta: fourierVec;
      fourierBOX: rBOX;

{To generate data with a frequency limit of w < 180, we generate an ordinary}
{ARMA sequence on a number of points denoted by EffectiveSize = SampleSize}
{X W/180. We proceed to find the full set of Fourier coefficients of this}
{sequence. Then we generate a set of SampleSize data points by Fourier Synthesis}

  begin
    {WRITELN('    BUGSTOP on entry to <GenerateBandLimitedData>');}
    {READLN;}

    pi := 4 * Arctan(1.0);
    effectiveSize := Trunc((dataBOX.NOPUNCT * ARMAGen.omega2Deg) / 180);

    ARMAGen.varepsilon := 1.0;
    fourierBOX.NOPUNCT := effectiveSize;
    GenerateData(ARMAGen, fourierBOX, dataVector);

    {WRITELN('    BUGSTOP in <GenerateBandLimitedData> after <GenerateData>');}
    {READLN; }

    fourierInc := (2.0 * pi) / effectiveSize;
    FourierCoefficients(fourierBOX, dataVector, fourierInc, alpha, beta);

    {WRITELN('    BUGSTOP in <GenerateBandLimitedData> after <FourierCoefficients>'); }
    {READLN;}


    Ncap := effectiveSize div 2;

    fourierInc := (2.0 * pi) / dataBOX.NOPUNCT;
    FourierSynthesis(dataBOX, Ncap, dataVector, fourierInc, alpha, beta);

    {WRITELN('    BUGSTOP in <GenerateBandLimitedData> after <FourierSynthesis>');}
    {READLN;}

    if ARMAGen.contaminated then
      begin
        for t := 0 to dataBOX.NOPUNCT - 1 do
          dataVector[t] := dataVector[t] + ARMAGen.noiseVar * whiteNoise(genSeed);
      end;

    thereIsData := true;
    pseudoData := true;
    ARMAGen.bandLimits := true;
    subsampledData := false;

  end; {GenerateBandLimitedData}

{________________________________________________}

  procedure AutoCovariances (Tcap, lags: integer;
                  var dataVector: longVector;
                  var aCovar: vector);

    var
      ybar: real;
      t, j: integer;

  begin {AutoCovariances}

  {Calculate the Mean}
    ybar := 0.0;
    for t := 0 to Tcap - 1 do
      ybar := ybar + dataVector[t];
    ybar := ybar / Tcap;

  {Calculate the AutoCovariances}
    for j := 0 to lags do
      begin {j}
        aCovar[j] := 0.0;
        for t := j to Tcap - 1 do
          aCovar[j] := aCovar[j] + (dataVector[t - j] - ybar) * (dataVector[t] - ybar);
        aCovar[j] := aCovar[j] / Tcap;
      end; {j}

  end; {AutoCovariances}

{________________________________________________}

  procedure CircularACovars (Tcap, lags: integer;
                  var dataVector: longVector;
                  var cirCovar: longvector);

    var
      ybar: real;
      t, tau, tPtau: integer;

  begin {AutoCovariances}

  {Calculate the Mean}
    ybar := 0.0;
    for t := 0 to Tcap - 1 do
      ybar := ybar + dataVector[t];
    ybar := ybar / Tcap;

  {Calculate the AutoCovariances}
    for tau := 0 to lags do
      begin {tau}
        cirCovar[tau] := 0.0;
        for t := 0 to Tcap - 1 do
          begin {t}
            tPtau := (t + tau) mod Tcap;
            cirCovar[tau] := cirCovar[tau] + (dataVector[tPtau] - ybar) * (dataVector[t] - ybar);
          end; {t}
        cirCovar[tau] := cirCovar[tau] / Tcap;
      end; {tau}

  end; {CircularACovars}

{________________________________________________}

  procedure YuleWalker (p, q: integer;
                  var gamma: vector;
                  var alpha: vector;
                  var varEpsilon: real);

    var
      A: matrix;
      b: vector;
      i, j, k: integer;

  begin {Yule-Walker}
    for i := 1 to p do
      begin {i}
        b[i] := -gamma[q + i];
        for j := 1 to p do
          begin {j}
            k := abs(q + i - j);
            A[i, j] := gamma[k];
          end{j}
      end; {i}

    LUsolve(1, p, A, alpha, b);
    alpha[0] := 1;
    varEpsilon := 0;
    for i := 0 to p do
      varEpsilon := varEpsilon + alpha[i] * gamma[i];

  end; {Yule-Walker}

{_________________________________________________}

  procedure LevinsonDurbin (var gamma: vector;
                  var ARMAEst: ARMARecord;
                  pacv: vector);

    var
      r, j, jstop, p: integer;
      c, g, store, varEpsilon: real;
      sigsqr, alpha: vector;

  begin
    p := ARMAEst.p;
    alpha[0] := 1.0;
    pacv[0] := 1.0;
    sigsqr[0] := gamma[0];
    r := 0;

    while r < p do
      begin {while}
        g := 0.0;
        for j := 0 to r do
          g := g + alpha[j] * gamma[r + 1 - j];
        c := g / sigsqr[r];

        jstop := r div 2;

        for j := 1 to jstop do
          begin {j}
            store := alpha[j];
            alpha[j] := store - c * alpha[r + 1 - j];
            alpha[r + 1 - j] := alpha[r + 1 - j] - c * store;
          end; {j}

        j := jstop + 1;
        if Odd(r) then
          alpha[j] := alpha[j] * (1 - c);

        alpha[r + 1] := -c;
        sigsqr[r + 1] := sigsqr[r] * (1 - sqr(alpha[r + 1]));

        r := r + 1;
        pacv[r] := -c;
      end; {while}

    varEpsilon := 0;
    for j := 0 to p do
      varEpsilon := varEpsilon + alpha[j] * gamma[j];

{Writeln('   varEpsilon = ', varEpsilon : 6 : 4);}

    ARMAEst.q := 0;
    ARMAEst.alpha := alpha;
    ARMAEst.mu[0] := 1.0;
    ARMAEst.varEpsilon := varEpsilon;
    ARMAEst.parametersFound := true;
    ARMAEst.rootsFound := false;
    ARestimates := true;

  end; {LevinsonDurbin}

{_________________________________________________}

  procedure MakeScreenBandSpec (var ARMAGen: ARMARecord;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

{This is the band-limited version of the data-generating spectrum}
{This routine serves to generate a vector of points that is used only}
{in the screen rendition of the spectrum.}

    var
      denominator, numerator: vector;
      pi, twoPi, increment, omega, scale: real;
      BOXWDT, SPECWDT, j: integer;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    SPECWDT := Round((BOXWDT * ARMAGen.omega2Deg) / 180);

    pi := 4.0 * Arctan(1.0);
    twoPi := 2.0 * pi;
    Laurent(ARMAGen.mu, numerator, ARMAGen.q);
    Laurent(ARMAGen.alpha, denominator, ARMAGen.p);
    scale := ARMAGen.varepsilon / twoPi;
    scale := scale * 180 / ARMAGen.omega2Deg;
    increment := pi / SPECWDT;

    spectrumBOX.yMax := 0.0;
    omega := 0.0;
    for j := 0 to SPECWDT do
      begin {j}
        spectrumVec[j] := Spectrum(scale, denominator, numerator, omega, ARMAGen.p, ARMAGen.q);
        omega := omega + increment;
        if spectrumVec[j] > spectrumBOX.yMax then
          spectrumBOX.yMax := spectrumVec[j];
      end; {j}

    spectrumBOX.yMin := 0.0;
    spectrumBOX.xMin := 0.0;
    spectrumBOX.xMax := pi;
{spectrumBOX.NOPUNCT := BOXWDT;}
 {This won't get used .It is dangerous}

  end; {MakeScreenBandSpec}

{_________________________________________________}

  procedure MakeScreenGenSpec (var ARMAGen: ARMARecord;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

{This is the data-generating spectrum supported on the interval [0, pi]}
{The routine serves to generate a vector of points that is used both in }
{the screen rendition and in the PostScript rendition}

{Observe that there is no alteration of the spectrumBOX}

{this is not specific to the screen. Therefore, the name should be changed:}
{MakeScreenGenSpec --> MakeDataGenSpec}

    var
      denominator, numerator: vector;
      scale, twoPi: real;
  begin
    twoPi := 8.0 * Arctan(1.0);
    Laurent(ARMAGen.mu, numerator, ARMAGen.q);
    Laurent(ARMAGen.alpha, denominator, ARMAGen.p);
    scale := ARMAGen.varepsilon / twoPi;
    SpectrumAndOmega(scale, denominator, numerator, ARMAGen.p, ARMAGen.q, spectrumBOX, SCREENBOX, spectrumVec);

{The max spectral value is recorded by SpectrumAndOmega in spectrumBOX.yMax}

  end;{MakeScreenGenSpec}

{_________________________________________________}

  procedure MakeAREstimSpec (var ARMAEst: ARMARecord;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

{This routine serves to generate a vector of points that is used both in}
{the screen rendition and in the PostScript rendition}

{Observe that there is no alteration of the spectrumBOX}

    var
      denominator, numerator: vector;
      scale, twoPi: real;

{We need to take account ofthe pre-specified zeros}

  begin
    twoPi := 8.0 * Arctan(1.0);

{numerator[0] := 1.0;}
    Laurent(ARMAEst.mu, numerator, ARMAEst.q);
    Laurent(ARMAEst.alpha, denominator, ARMAEst.p);

    scale := ARMAEst.varepsilon / twoPi;
    SpectrumAndOmega(scale, denominator, numerator, ARMAEst.p, ARMAEst.q, spectrumBOX, SCREENBOX, spectrumVec);

{The max spectral value is recorded by SpectrumAndOmega in spectrumBOX.yMax}

  end;{MakeAREstimSpec}

 {________________________________________________}

  procedure MakePostBandSpec (var ARMAGen: ARMARecord;
                  var SCREENBOX: IBOX;
                  var specVec: longVector;
                  var specBOX: rBOX);

{This procedure generates the coordinates of the PostScript rendition of}
{the band-limited spectrum. It exploits the higher resolution of PostScript}
{to acheive a plot that is finer than the screen version.}

    var
      denominator, numerator: vector;
      pi, twoPi, increment, omega, scale: real;
      bandFraction: real;
      BOXWDT, SPECWDT, resolution: integer;
      j: integer;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    SPECWDT := Round((BOXWDT * ARMAGen.omega2Deg) / 180);
    bandFraction := ARMAGen.omega2Deg / 180;

    resolution := 1;
    if bandFraction < 0.5 then
      resolution := 2;
    if bandFraction < 0.25 then
      resolution := 4;

    pi := 4.0 * Arctan(1.0);
    twoPi := 2.0 * pi;
    Laurent(ARMAGen.mu, numerator, ARMAGen.q);
    Laurent(ARMAGen.alpha, denominator, ARMAGen.p);
    scale := ARMAGen.varepsilon / twoPi;
    scale := scale * 180 / ARMAGen.omega2Deg;
    increment := pi / (SPECWDT * resolution); {Diminish this to give more resolution}

    specBOX.yMax := 0.0;
    omega := 0.0;
    for j := 0 to SPECWDT * resolution do
      begin {j}
        specVec[j] := Spectrum(scale, denominator, numerator, omega, ARMAGen.p, ARMAGen.q);
        omega := omega + increment;
        if specVec[j] > specBOX.yMax then
          specBOX.yMax := specVec[j];
      end; {j}

    specBOX.yMin := 0.0;
    specBOX.xMin := 0.0;
    specBOX.xMax := pi;
    specBOX.yLB := 0.0;
    specBOX.NOPUNCT := SPECWDT * resolution; {This will get used}

  end; {MakePostBandSpec}

{_________________________________________________}

  procedure ARMACovariances (var ARMAGen: ARMARecord;
                  var gamma: vector);

    var
      alpha, mu: vector;
      varEpsilon: real;
      lags, p, q: integer;
      i, j, t, tau, r: integer;
      delta, f: vector;
      a: matrix;

{Find the delta values}
  begin
    varEpsilon := ARMAGen.varepsilon;
    p := ARMAGen.p;
    q := ARMAGen.q;
    alpha := ARMAGen.alpha;
    mu := ARMAGen.mu;
    r := Max(p, q);
    lags := r;

    for t := 0 to q do
      begin
        delta[t] := mu[t] * varEpsilon;
        tau := min(t, p);
        for i := 1 to tau do
          delta[t] := delta[t] - delta[t - i] * alpha[i];
        delta[t] := delta[t] / alpha[0]
      end;

    for i := 0 to r do
      for j := 0 to r do
        begin {i,j : form the matrix of alpha values}
          a[i, j] := 0.0;
          if ((i - j) >= 0) and ((i - j) <= p) then
            a[i, j] := alpha[i - j];
          if ((i + j) <= p) and (j > 0) then
            a[i, j] := a[i, j] + alpha[i + j];
        end; {i, j}

    for i := 0 to r do
      begin{i : form the RHS vector}
        f[i] := 0.0;
        for j := i to q do
          f[i] := f[i] + mu[j] * delta[j - i]
      end; {i}

{Solve for the initial autocovariances}
    LUsolve(0, r + 1, a, gamma, f);

{Find the succeeding autocovariances}
    for i := r + 1 to lags do
      begin {i}
        gamma[i] := 0.0;
        for j := 1 to p do
          gamma[i] := gamma[i] - alpha[j] * gamma[i - j];
        gamma[i] := gamma[i] / alpha[0];
      end; {i}

    ARMAGen.yVar := gamma[0];

  end; {ARMACovariances}

 {________________________________________________}

  procedure ImposeZeros (var ARMAEst: ARMARecord);

    var
      theMenuItem: integer;

  begin
    ClrScr;

    Writeln;
    Writeln('        IMPOSE MOVING-AVERAGE ZEROS ');
    Writeln;
    Writeln('        Zeros may be imposed at the zero frequency or at the Nyquist');
    Writeln('        frequency or at both frequencies');
    Writeln;
    Writeln;
    Writeln('        1. Impose a zero at zero frequency');
    Writeln;
    Writeln('        2. Impose a zero at the Nyquist frequency');
    Writeln;
    Writeln('        3. Impose a zeros the zero frequency and the Nyquist frequency');
    Writeln;
    Writeln('                        Select a number and press <RETURN>');

    GetBoundedInteger(1, 3, theMenuItem);

    case theMenuItem of
      1: 
        begin
          ARMAEst.q := 1;
          ARMAEst.mu[0] := 1.0;
          ARMAEst.mu[1] := -1.0;
        end;
      2: 
        begin
          ARMAEst.q := 1;
          ARMAEst.mu[0] := 1.0;
          ARMAEst.mu[1] := 1.0;
        end;
      3: 
        begin
          ARMAEst.q := 2;
          ARMAEst.mu[0] := 1.0;
          ARMAEst.mu[1] := 0.0;
          ARMAEst.mu[2] := -1.0;
        end;
    end; {cases}

    PreSpecifiedZeros := true;

  end; {ImposeZeros}

{_________________________________________________}

  procedure GetPsivar (var ARMAEst: ARMARecord;
                  var aCovar: vector);

{This procedure finds the variance of the forcing function in the case}
{where there is a prespecified MA component such as a component that}
{imposes zeros at the zero frequency and/or at the Nyquait frequency.}

    var
      i, j, k, p, q: integer;
      muSumSqr, psiVar: real;

  begin
    p := ARMAEst.p;
    q := ARMAEst.q;

    psiVar := 0.0;
    for i := 0 to p do
      for j := 0 to p do
        begin
          k := Abs(j - i);
          psiVar := psiVar + ARMAEst.alpha[i] * ARMAEst.alpha[j] * aCovar[k];
        end;

    muSumSqr := 1.0;
    for j := 1 to q do
      muSumSqr := muSumSqr + ARMAEst.mu[j] * ARMAEst.mu[j];
    ARMAEst.varEpsilon := psiVar / muSumSqr;

  end; {GetPsivar}

{_________________________________________________}

  procedure PoleZeroEstimates (var ARMAEst: ARMARecord);

    var
      j: integer;
      cxAlpha, cxMu: complexVector;

  begin
    for j := 0 to ARMAEst.p do
      begin
        cxAlpha[j].re := ARMAEst.alpha[j];
        cxAlpha[j].im := 0.0;
      end;
    for j := 0 to ARMAEst.q do
      begin
        cxMu[j].re := ARMAEst.mu[j];
        cxMu[j].im := 0.0;
      end;

    FindRoots(ARMAEst.p, cxAlpha, ARMAEst.poles, ARMAEst.polarPoles);
    FindRoots(ARMAEst.q, cxMu, ARMAEst.zeros, ARMAEst.polarZeros);

  end; {PoleZeroEstimates}

{_________________________________________________}

  procedure ARMAparameters (var ARMAEst: ARMARecord;
                  var aCovar: vector);

    var
      p, q, t, i, j, k: integer;
      psi: vector;

  begin {ARMAparameters}
    p := ARMAEst.p;
    q := ARMAEst.q;

    YuleWalker(p, q, aCovar, ARMAEst.alpha, ARMAEst.varEpsilon);

{if q = 0 then}
{WRITELN(' the pure AR varEpsilon is ', ARMAEst.varEpsilon : 6 : 4);}

    if ((q > 0) and not PreSpecifiedZeros) then
      begin {Unrestricted MA}
        for t := 0 to q do
          begin {t}
            psi[t] := 0.0;
            for i := 0 to p do
              for j := 0 to p do
                begin {i, j}
                  k := Abs(t + j - i);
                  psi[t] := psi[t] + ARMAEst.alpha[i] * ARMAEst.alpha[j] * aCovar[k];
                end; {i, j}
          end;{t}

        Minit(ARMAEst.mu, ARMAEst.varEpsilon, psi, q)
      end; {Unrestricted MA}

    if PreSpecifiedZeros then
      GetPsivar(ARMAEst, aCovar);

  end;{ARMAparameters}

{_________________________________________________}

  procedure ModelAnalysis (var ARMAGen, ARMAEst: ARMARecord;
                  var dataBOX: rBOX);

    var
      j: integer;

  begin
    ClrScr;

    if ARMAModel then
      begin {ARMAModel}
        Writeln;
        Writeln('    THE PARAMETERS OF THE SPECIFIED MODEL');
        Writeln;
        Writeln('    The Autoregressive Parameters of the Specified Model');
        Writeln;
        for j := 0 to ARMAGen.p do
          Writeln('    alpha[', j : 1, '] = ', ARMAGen.alpha[j] : 6 : 4);
        Writeln;
        for j := 1 to ARMAGen.p do
          Writeln('    poles[', j : 1, '].re = ', ARMAGen.poles[j].re : 5 : 5, '   poles[', j : 1, '].im = ', ARMAGen.poles[j].im : 5 : 5);
        Writeln;
        for j := 1 to ARMAGen.p do
          begin
            Write('    root(', j : 1, ').modu = ', ARMAGen.Polarpoles[j].modu : 8 : 4);
            Writeln('    root(', j : 1, ').argu = ', ARMAGen.Polarpoles[j].argu : 8 : 4);
          end;
        Writeln;

        Writeln(' ' : 59, '<RETURN>');
        Readln;
        if (ARMAGen.q > 0) then
          begin {MA parameters}
            Writeln('    The Moving-Average Parameters of the Specified Model');
            Writeln;
            for j := 0 to ARMAGen.q do
              Writeln('    mu[', j : 1, '] = ', ARMAGen.mu[j] : 6 : 4);
            Writeln;
            for j := 1 to ARMAGen.q do
              begin
                Write('    zeros[', j : 1, '].re = ', ARMAGen.zeros[j].re : 5 : 5);
                Writeln('    zeros[', j : 1, '].im = ', ARMAGen.zeros[j].im : 5 : 5);
              end;
            Writeln;
            for j := 1 to ARMAGen.q do
              begin
                Write('    root(', j : 1, ').modu = ', ARMAGen.Polarzeros[j].modu : 8 : 4);
                Writeln('    root(', j : 1, ').argu = ', ARMAGen.Polarzeros[j].argu : 8 : 4);
              end;
            Writeln;
            Writeln(' ' : 59, '<RETURN>');
            Readln;
          end; {MA Parameters}
      end; {ARMAModel}

    ClrScr;

    if ARestimates then
      begin {ARestimates}
        Writeln;
        Writeln('    THE PARAMETERS OF THE ESTIMATED MODEL');
        Writeln;
        Writeln('    The Autoregressive Parameters of the Estimated Model');
        Writeln;
        for j := 0 to ARMAEst.p do
          Writeln('    alpha[', j : 1, '] = ', ARMAEst.alpha[j] : 6 : 4);
        Writeln;
        for j := 1 to ARMAEst.p do
          begin
            Write('    poles[', j : 1, '].re = ', ARMAEst.poles[j].re : 5 : 5);
            Writeln('    poles[', j : 1, '].im = ', ARMAEst.poles[j].im : 5 : 5);
          end;
        Writeln;
        for j := 1 to ARMAEst.p do
          begin
            Write('    root(', j : 1, ').modu = ', ARMAEst.Polarpoles[j].modu : 8 : 4);
            Writeln('    root(', j : 1, ').argu = ', ARMAEst.Polarpoles[j].argu : 8 : 4);
          end;
        Writeln;
        Writeln(' ' : 59, '<RETURN>');
        Readln;

        if (ARMAEst.q > 0) then
          begin {MA parameters}
            Writeln('    The Moving-Average Parameters of the Estimated Model');
            Writeln;
            for j := 0 to ARMAEst.q do
              Writeln('    mu[', j : 1, '] = ', ARMAEst.mu[j] : 6 : 4);
            Writeln;
            for j := 1 to ARMAEst.q do
              begin
                Write('    zeros[', j : 1, '].re = ', ARMAEst.zeros[j].re : 5 : 5);
                Writeln('    zeros[', j : 1, '].im = ', ARMAEst.zeros[j].im : 5 : 5);
              end;
            Writeln;
            for j := 1 to ARMAEst.p do
              begin
                Write('    root(', j : 1, ').modu = ', ARMAEst.Polarzeros[j].modu : 8 : 4);
                Writeln('    root(', j : 1, ').argu = ', ARMAEst.Polarzeros[j].argu : 8 : 4);
              end;
            Writeln;
            Writeln(' ' : 59, '<RETURN>');
            Readln;
          end; {MA Parameters}
      end; {ARestimates}

  end; {ModelAnalysis}

{________________________________________________}

end. {AUTOREG: Unit 9 of CONGRESS.PAS}

