unit DATA; {Unit 10 of CONGRESS.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS, SCREEN, POST, ACQUIRE, SPECTRAL, AUTOREG, Dos, Graph, Crt;

  procedure PlotTheData (var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure LogOfVector (var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure DiffOfVector (var diffOrder: integer;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure DataCumulation (var dataBOX: rBOX;
                  var diffBOX: rBOX;
                  diffOrder: integer;
                  var dataVector, diffVector: longVector);

  procedure TransferTheData (var diffOrder: integer;
                  var dataBOX, trendBOX: rBOX;
                  var dataVector, trendVector: longVector);

  procedure TestForStationarity (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var abortit: boolean);

  procedure Decimate (step: integer;
                  var dataBOX: rBOX;
                  var dataVector: longVector);
{This is called by procedure <<SubsampleTheData>>}

  procedure SubsampleTheData (var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure ResampleTheData (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var FrequencyLimit: real);

  procedure GetARMAData (var ARMAGen: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure GetBandLimitedData (var ARMAGen: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure FastPeriodogram (var dataBOX, periodoBOX: RBOX;
                  var dataVector, periodoVec: longVector);

  procedure PolyRegress (x, y: longVector;
                  var alpha, gamma, delta: vector;
                  var poly: longVector;
                  q, n: integer);

  procedure OrthoToPower (alpha, gamma, delta: vector;
                  var beta: vector;
                  q: integer);

  function PolyOrdinate (z: real;
                  var alpha, gamma, delta: vector;
                  q: integer): real;

  procedure MakePolyTrend (polyOrder: integer;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec: longVector);

  procedure FitPolyTrend (var dataBOX, trendBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec: longVector);

  procedure LowpassComponent (var dataBOX, trendBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec: longVector);

implementation

  {_________________________________________________}

  procedure PlotTheData (var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);
    var
      legend: string;

  begin
    legend := ' ';

    if (thereIsData = false) then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
      end;

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    ScreenMakePlot(dataBOX, SCREENBOX, dataVector, legend);

    if (PSPict = true) then
      begin
        WritePostFile(dataBOX, POSTBOX, dataVector, legend);
      end;

  end; {PlotTheData}

{________________________________________________}

  procedure LogOfVector (var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      i: integer;
      yMax, yMin, yMean: real;

  begin
    if logsTaken then
      begin
        ClrScr;
        Writeln;
        Sysbeep(3);
        Writeln('    TAKE LOGS OF THE DATA');
        Writeln;
        Writeln('    The logarithms have already been taken!');
        Writeln(' ' : 60, '<RETURN>');
        Readln;
      end
    else if (dataBOX.yMin <= 0.0) then
      begin {elseif}
        ClrScr;
        Writeln;
        Sysbeep(3);
        Writeln('    TAKE LOGS OF THE DATA');
        Writeln;
        Writeln('    The data contains elements with values which are zero or negative.');
        Writeln;
        Writeln('    The operation of taking logarithms of the data must be aborted');
        logsTaken := false;
        Writeln;
        Writeln(' ' : 64, '<RETURN>');
        Readln;
      end {elseif}
    else if (thereIsData = false) then
      begin  {elseif}
        Writeln;
        Sysbeep(3);
        Writeln('    TAKE LOGS OF THE DATA');
        Writeln;
        Writeln('    THere is no data!');
        Writeln(' ' : 60, '<RETURN>');
        Readln;
      end {elseif}
    else
      begin {else}
        yMean := 0.0;
        for i := 0 to dataBOX.NOPUNCT - 1 do
          begin {i}
            dataVector[i] := ln(dataVector[i]);
            if i = 0 then
              begin
                yMax := dataVector[0];
                yMin := dataVector[0];
              end;
            yMean := yMean + dataVector[i];
            if dataVector[i] > yMax then
              yMax := dataVector[i];
            if dataVector[i] < yMin then
              yMin := dataVector[i];
          end; {i}

        yMean := yMean / dataBOX.NOPUNCT;
        logsTaken := true;
        dataBOX.yMax := yMax; {the maximum value of y in the Data vector}
        dataBOX.yMin := yMin; {the minimum value of y in the Data vector}
        dataBOX.yMean := yMean;
      end; {else}

  end; {LogOfVector}

{_______________________________________________}

  procedure DiffOfVector (var diffOrder: integer;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      t: integer;

  begin
    for t := 0 to dataBOX.NOPUNCT - 2 do
      dataVector[t] := dataVector[t + 1] - dataVector[t];
    dataBOX.NOPUNCT := dataBOX.NOPUNCT - 1;

    MakeDataBox(dataBOX, dataVector);

    diffOrder := diffOrder + 1;

  end; {DiffOfVector}

 {________________________________________________}

  procedure DataCumulation (var dataBOX: rBOX;
                  var diffBOX: rBOX;
                  diffOrder: integer;
                  var dataVector, diffVector: longVector);

    var
      t: integer;
      sum0, sum1: real;
      det, s11, s12, s21, s22, b11, b12, b21, b22, Tcap: real;
      diff, zStar0, zStar1: real;

  begin
    if (diffOrder = 1) then
      begin
        for t := 1 to diffBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + diffVector[t - 1];

        for t := diffBOX.NOPUNCT downto 1 do {Shift the registers}
          diffVector[t] := diffVector[t - 1];
        diffVector[0] := 0;

        sum0 := 0.0;
        for t := 0 to dataBOX.NOPUNCT - 1 do
          sum0 := sum0 + dataVector[t] - diffVector[t];

        zStar0 := sum0 / dataBOX.NOPUNCT;

        for t := 0 to dataBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + zStar0;
      end; {diffOrder = 1}

    if (diffOrder = 2) then
      begin
        diffVector[1] := diffVector[1] + 2 * diffVector[0];
        for t := 2 to diffBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + 2 * diffVector[t - 1] - diffVector[t - 2];

        for t := diffBOX.NOPUNCT + 1 downto 2 do {Shift the registers}
          diffvector[t] := diffvector[t - 2];
        diffvector[1] := 0;
        diffvector[0] := 0;

        sum0 := dataVector[0];
        sum1 := 0.0;
        for t := 1 to dataBOX.NOPUNCT - 1 do
          begin {t}
            diff := dataVector[t] - diffVector[t];
            sum0 := sum0 + (t + 1) * diff;
            sum1 := sum1 + t * diff;
          end; {t}

        Tcap := dataBOX.NOPUNCT;

        s11 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0;
        Tcap := Tcap - 1;
        s12 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0 + Tcap * (Tcap + 1) / 2;
        s21 := s12;
        s22 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0;
        det := s11 * s22 - s12 * s21;

        b11 := s22 / det;
        b12 := -s12 / det;
        b21 := -s21 / det;
        b22 := s11 / det;

        zStar0 := b11 * sum0 + b12 * sum1;
        zStar1 := b21 * sum0 + b22 * sum1;

        for t := 0 to dataBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + zStar0 * (t + 1) + zStar1 * t;
      end; {diffOrder = 2}

  end; {DataCumulation}

{_________________________________________________}

  procedure TransferTheData (var diffOrder: integer;
                  var dataBOX, trendBOX: rBOX;
                  var dataVector, trendVector: longVector);

{Observe that the dataVector is unaffected and,}
{therefore, the dataDiffOrder is unchanged. }

    var
      d: integer;

  begin
    trendVector := dataVector;
    trendBOX := dataBOX;

    d := dataDiffOrder;
    while (d < diffOrder) do
      begin
        DiffOfVector(d, trendBOX, trendVector);
      end; {d = diffOrder}

  end; {TransferTheData}

 {________________________________________________}

  procedure TestForStationarity (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var abortit: boolean);
    var
      slope, tValue, a, b: real;
      tString: string;

  begin
    ClrScr;

    Writeln;
    Writeln('    TEST FOR STATIONARITY');
    Writeln;
    Writeln('    Before  subampling the data, you  may wish to reduce the data to');
    Writeln('    stationarity using a polynomial regression. To determine whether');
    Writeln('    this is necessary, the statistical significance of the estimated');
    Writeln('    slope parameter of the data is evaluated via a t-test.');

    a := 1.15;  { This demarcates an upper tail = 12.25 percent}
    b := 1.3;  { This demarcates an upper tail = 10.00 percent}

    TrendTest(slope, tValue, dataBOX, dataVector);
    Writeln;
    tString := TalyToString(tValue);
    Writeln('    The t-value of the slope coefficient is t = ', tString);
    Writeln;

    abortIt := false;
    if (tValue > a) and (tValue < b) then
      begin
        Writeln('    You are advised to extract a trend.');
        YesOrNo('   Will you take the advice to extract a trend', abortIt);
      end
    else if (tValue >= b) then
      begin
        Writeln('    Before proceeding you must remove the trend');
        abortIt := true;
      end;

    Writeln(' ' : 60, '<RETURN>');
    Readln;

  end; {TestForStationarity}

{________________________________________________}

  procedure Decimate (step: integer;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

{This is called by procedure <<SubsampleTheData>>}

    var
      t, tau: integer;
  begin
    tau := 0;
    with dataBOX do
      begin {with dataBOX}
        yMean := 0.0;
        yMax := dataVector[0];
        yMin := dataVector[0];
        xMax := 0.0;
        xMin := 0.0;
        for t := 0 to NOPUNCT - 1 do
          begin {t}
            if (t mod step) = 0 then
              begin {if}
                dataVector[tau] := dataVector[t];
                tau := tau + 1;
                yMax := Rmax(yMax, dataVector[t]);
                yMin := Rmin(yMin, dataVector[t]);
                yMean := yMean + dataVector[t];
              end; {if}
          end; {t}
        NOPUNCT := tau;
        yMean := yMean / NOPUNCT;
        xMin := 0.0;
        xMax := NOPUNCT - 1;
        yUB := yMax;
        yLB := 0.0;
        xLB := xMin;
        xRB := xMax;
      end; {with dataBOX}

  end; {Decimate}

{________________________________________________}

  procedure SubsampleTheData (var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      abortIt: boolean;
      choice: char;
      SampleStep: integer;
      FrequencyLimit: real;

  begin
    if (not thereIsData) then
      ReadTheData(dataBOX, dataVector);

    abortit := false;
    if (trendEstimated = false) and empiricData then
      TestForStationarity(dataBOX, dataVector, abortit);

    if (abortIt = false) then
      begin {continue}
        ClrScr;

        Writeln;
        Writeln('    SUBSAMPLE THE DATA');
        Writeln;
        Writeln('    The data may be subsampled directly by taking one in n of the data points,');
        Writeln('    where n is an integer');
        Writeln;
        Writeln('    Alternatively, a  continuous trajectory can be reconstituted from the data');
        Writeln('    which can be resampled by taking points separated by an arbitrary interval');
        Writeln;
        Writeln('    Please choose from the following options:');
        Writeln;
        Writeln('    (a)  Subsample the data, taking one in n points');
        Writeln;
        Writeln('    (b)  Resample from the continuous trajectory');
        Writeln;
        Readln(choice);

        if (choice in ['a', 'A']) then
          begin
            Writeln('    (a)  To subsample the data, select every  nth point, choosing n in [2, 8].');
            Writeln('         Select 2, 3, 4 etc or type 1 to avoid subsampling.');
            GetBoundedInteger(1, 8, SampleStep);
            if SampleStep in [2, 3, 4, 5, 6, 7, 8] then
              Decimate(SampleStep, dataBOX, dataVector);
          end;

        if (choice in ['b', 'B']) then
          begin
            Writeln('    (b)  Choose  a frequency value w in degrees  within the interval (0, 180]');
            Writeln('         to represent the maximum frequency within the data. The interval  at');
            Writeln('         which the continuous  trajectory will  be sampled is of 180/w units.');
            Writeln('         To avoid resampling, type 0.');
            repeat
              ReadANumber(FrequencyLimit);
              if (FrequencyLimit > 180.0) then
                begin
                  Sysbeep(3);
                  Writeln('         The frequency limit must be less that the Nyquist frequency of 90 degrees')
                end;
            until (FrequencyLimit < 180);
            if FrequencyLimit > 0 then
              ResampleTheData(dataBOX, dataVector, FrequencyLimit);
          end;

        if (SampleStep > 1) or (FrequencyLimit > 0) then
          begin
            SubsampledData := true;
            AREstimates := false;
          end;

        if (dataBOX.NOPUNCT > 128) then
          slowOrFast := fast
        else
          slowOrFast := slow;

      end; {continue, since abortit = false}

  end; {SubsampleTheData}

{________________________________________________}

  procedure ResampleTheData (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var FrequencyLimit: real);

{This procedure resamples the data by reconstituting a continuous trajectory }
{from  a  data sequence limited in frequency to less than the Nyquist frequency}
{of pi radians per sample interval. The frequency limit must be specified by the}
{user}

{First we derive the Fourier Coefficients of the original data sequence. Then}
{we synthesize the re-sampled data points from a set of Fourier coefficients}
{bounded by the frequency limit. The number of data points equals the number}
{of coefficients.}

    var
      alpha, beta: fourierVec;
      pi, fourierInc: real;
      Ncap, effectiveSize: integer;

  begin
    pi := 4 * Arctan(1.0);
    fourierInc := (2.0 * pi) / dataBOX.NOPUNCT;
    FourierCoefficients(dataBOX, dataVector, fourierInc, alpha, beta);

    effectiveSize := Trunc((dataBOX.NOPUNCT * FrequencyLimit) / 180);
    Ncap := effectiveSize div 2;
    dataBOX.NOPUNCT := effectiveSize;
    fourierInc := (2.0 * pi) / dataBOX.NOPUNCT;
    FourierSynthesis(dataBOX, Ncap, dataVector, fourierInc, alpha, beta);

  end; {ResampleTheData}
{________________________________________________}

  procedure GetARMAData (var ARMAGen: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      stability: boolean;

  begin

    if ARMAmodel then
      begin
        ClrScr;
        Writeln;
        Writeln('    GENERATE THE ARMA DATA');
        Writeln;
        SysBeep(3);
        YesOrNo('    You have already specified the model. Do you wish to retain it', ARMAModel);
      end;

    stability := true;
    if (ARMAModel = false) then
      begin
        RationalAcquisition(ARMAGen, stability);

        ClrScr;

        Writeln;
        Writeln('    GENERATE THE ARMA DATA');
      end;

    Writeln;

    ARMAGen.varepsilon := 1.0;
    Writeln('    What is size T of the data sample you wish to generate?');
    GetBoundedInteger(16, 500, dataBOX.NOPUNCT);
    GenerateData(ARMAGen, dataBOX, dataVector); {in AUTOREG}

    thereIsData := true;
    pseudoData := true;
    empiricData := false;

    trendEstimated := false;
    SubsampledData := false;

    ARestimates := false;
    ARMAGen.bandLimits := false;
    ARMAGen.contaminated := false;


{ARMAGen.rootsFound := true;}
{parametersFound := true;}
{simData := true;}
{empiricData := false;}

{ARMAEst.parametersFound := false;}
{ARMAEst.rootsFound := false;}
{dataAlert := false;}

  end; {GetARMAData}

{_________________________________________________}

  procedure GetBandLimitedData (var ARMAGen: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      omegaCut, varFraction, noiseVar: real;
      sampleSize, effectiveSize: integer;
      gamma: vector;
      contaminate: boolean;

  begin

    if ARMAmodel then
      begin
        ClrScr;
        Writeln;
        Writeln('    BAND-LIMITED DATA');
        Writeln;
        SysBeep(3);
        YesOrNo('    You have already specified the model. Do you wish to retain it', ARMAModel);
      end;

    if (ARMAModel = false) then
      begin
        RationalAcquisition(ARMAGen, stability);
        ClrScr;
        Writeln;
        Writeln('    GENERATE THE ARMA DATA');
      end;

    Writeln;
    Writeln('    Specify the upper limit of the frequency band in degrees');
    ReadBoundedNumber(0.0, 180.0, omegaCut);
    ARMAGen.bandLimits := true;
    ARMAGen.omega2Deg := omegaCut;

    Writeln('    How many data points do you require');

    GetBoundedInteger(0, 512, sampleSize);
    dataBOX.NOPUNCT := sampleSize;

    effectiveSize := Trunc((sampleSize * omegaCut) / 180);
    Writeln('    The effective sample size for the band-limited process is ', effectiveSize : 3);

    Writeln;
    Writeln('    The band limited sample will be generated by taking ', sampleSize : 2, ' sample');
    Writeln('    points  from  a  continuous  function  constituted  from  the');
    Writeln('    Fourier  ordinates  of a sample of ', effectiveSize : 2, ' points generated  by  an');
    Writeln('    ordinary ARMA process supported  on the Nyquist interval which');
    Writeln('    runs from 0 to pi radians per sample interval.');
    Writeln;
    contaminate := false;
    YesOrNo('    Do you wish to add a white-noise contamination to the data', contaminate);

    if contaminate then
      begin {contaminate}
        Writeln('    Specify the variance of the contamination as a  proportion');
        Writeln('    of the variance of the ARMA process');
        ReadBoundedNumber(0.0, 1.0, varFraction);
        Writeln(' ' : 58, '<RETURN>');
        Readln;
      end; {contaminate}

    ClrScr;

    if contaminate then
      begin {contaminate}
        ARMAGen.contaminated := true;
        ARMACovariances(ARMAGen, gamma);
        noiseVar := varFraction * gamma[0];
        ARMAGen.noiseVar := noiseVar;
      end; {contaminate}

    GenerateBandLimitedData(ARMAGen, dataBOX, dataVector);

    thereIsData := true;
    pseudoData := true;
    empiricData := false;
    trendEstimated := false;
    ARestimates := false;
    ARMAGen.bandLimits := true;

  end; {GetBandLimitedData}

{________________________________________________}

  procedure FastPeriodogram (var dataBOX, periodoBOX: RBOX;
                  var dataVector, periodoVec: longVector);

    var
      x, xi: longVector;
      t, Tcap, Ncap: integer;
      maxi: real;

  begin
    if dataBOX.NOPUNCT >= 16 then
      begin {Calculate the Peridogram}
        for t := 0 to dataBOX.NOPUNCT - 1 do
          begin
            x[t] := dataVector[t];
            if empiricData then
              x[t] := x[t] - dataBox.yMean;
          end;
      end;
{We Have already calculated yMean within ReadDataSafely}

    Tcap := dataBOX.NOPUNCT;
    RealMixedRadix(x, xi, Tcap);
    Ncap := Tcap div 2;

{RealMixedRadix presumes that the sample size is even }

    Periodogram(xi, periodoVec, maxi, Tcap);

    periodoBOX.xMax := pi;
    periodoBOX.xMin := 0;
    periodoBox.yMax := maxi;
    periodoBOX.yMin := 0;
    periodoBox.NOPUNCT := Ncap;

  end;{FastPeriodogram}

{________________________________________________}

  procedure PolyRegress (x, y: longVector;
                  var alpha, gamma, delta: vector;
                  var poly: longVector;
                  q, n: integer);

    var
      phi, philag: longVector;
      denom, lagdenom, temp: real;
      k, t, Tcap: integer;

  begin {PolyRegress}

    alpha[0] := 0.0;
    gamma[1] := 0.0;
    Tcap := n + 1;

    for t := 0 to n do
      begin
        alpha[0] := alpha[0] + y[t];
        gamma[1] := gamma[1] + x[t];
      end;

    alpha[0] := alpha[0] / Tcap;
    gamma[1] := gamma[1] / Tcap;
    lagdenom := Tcap;

    for t := 0 to n do
      begin
        philag[t] := 1;
        phi[t] := x[t] - gamma[1];
        poly[t] := alpha[0];
      end;

    for k := 1 to q do
      begin {k}

        alpha[k] := 0.0;
        gamma[k + 1] := 0;
        denom := 0.0;

        for t := 0 to n do
          begin {t}
            alpha[k] := alpha[k] + y[t] * phi[t];
            denom := denom + Sqr(phi[t]);
            gamma[k + 1] := gamma[k + 1] + Sqr(phi[t]) * x[t];
          end; {t}

        alpha[k] := alpha[k] / denom;
        gamma[k + 1] := gamma[k + 1] / denom;
        delta[k + 1] := denom / lagdenom;
        lagdenom := denom;

        for t := 0 to n do
          begin {t}
            poly[t] := poly[t] + alpha[k] * phi[t];
            temp := phi[t];
            phi[t] := (x[t] - gamma[k + 1]) * phi[t] - delta[k + 1] * philag[t];
            philag[t] := temp;
          end; {t}

      end; {k}

  end; {PolyRegress}

{________________________________________________}

  procedure OrthoToPower (alpha, gamma, delta: vector;
                  var beta: vector;
                  q: integer);

    var
      phiplus: real;
      phi, philag: vector;
      i, k: integer;

  begin
    phi[-1] := 0;
    phi[0] := -gamma[1];
    phi[1] := 1; {redundant}
    philag[0] := 1;
    philag[1] := 0;
    beta[0] := alpha[0];

{find the power-form coefficients}
    for k := 1 to q do
      begin {k}
        beta[k] := 0.0;
        for i := k downto 0 do
          begin
            beta[i] := beta[i] + alpha[k] * phi[i];
            phiplus := phi[i - 1] - gamma[k + 1] * phi[i] - delta[k + 1] * philag[i];
            philag[i] := phi[i];
            phi[i] := phiplus;
          end;
        phi[k + 1] := 1;
        philag[K + 1] := 0;
      end;{k}

  end; {OrthoToPower}

{________________________________________________}

  function PolyOrdinate (z: real;
                  var alpha, gamma, delta: vector;
                  q: integer): real;

    var
      d: vector;
      j: integer;

  begin
    d[q] := alpha[q];
    d[q - 1] := alpha[q - 1] + d[q] * (z - gamma[q]);
    for j := q - 2 downto 0 do
      d[j] := alpha[j] + d[j + 1] * (z - gamma[j + 1]) - d[j + 2] * delta[j + 2];
    PolyOrdinate := d[0];

  end; {PolyOrdinate}

{________________________________________________}

  procedure MakePolyTrend (polyOrder: integer;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec: longVector);

    var
      minResidual, maxResidual, trendMean, residueMean: real;
      alpha, gamma, delta: vector;
      x: longVector;
      i, q: integer;
{NB poly contains the ordinstes at the data points}
{We might wish to have the values of the polynomial at other points}
{these should be  stored in trendVec}

  begin
    q := polyOrder;
    for i := 0 to dataBOX.NOPUNCT - 1 do
      x[i] := i;

    PolyRegress(x, dataVector, alpha, gamma, delta, trendVector, q, dataBOX.NOPUNCT - 1);

    trendMean := 0.0;
    residueMean := 0.0;
{Make Residuals Vector: residueVec has the dimension of NOPUNCT}
    for i := 0 to dataBOX.NOPUNCT - 1 do
      begin {i}
        residueVec[i] := dataVector[i] - trendVector[i];
        residueMean := residueMean + residueVec[i];
        trendMean := trendMean + trendVector[i];
        if i = 0 then
          begin
            minResidual := residueVec[0];
            maxResidual := residueVec[0];
          end;
        maxResidual := Rmax(maxResidual, residueVec[i]);
        minResidual := Rmin(minResidual, residueVec[i]);
      end; {i}
    residueMean := residueMean / dataBOX.NOPUNCT;
    trendMean := trendMean / dataBOX.NOPUNCT;

    residueBOX := dataBOX;
    trendBOX := dataBOX;
    residueBOX.yMin := minResidual;
    residueBOX.yMax := maxResidual;
    residueBOX.yMean := residueMean;
    trendBOX.yMean := trendMean;

    trendEstimated := true;
  end; {MakePolyTrend}

{_______________________________________________}

  procedure FitPolyTrend (var dataBOX, trendBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec: longVector);

    var
      q: integer;
      legend: string;
      replace: boolean;

  begin

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetup(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}
      end;

    if (thereIsData = false) then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector); {in UTILS}
      end;

    ClrScr;
    Writeln;
    Writeln('     FIT A POLYNOMIAL TREND');
    Writeln;

    Writeln;
    Writeln('    What is the degree of the polynomial?');
    GetBoundedInteger(0, 12, q);

    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrame(dataBOX, SCREENBOX);
    ScreenPlot(dataBOX, SCREENBOX, dataVector);

    trendBOX := dataBOX;
    trendBOX.xMin := 0;
    residueBOX := trendBOX;
    MakePolyTrend(q, dataBOX, trendBox, residueBOX, dataVector, trendVector, residueVec);

    ScreenPlot(trendBOX, SCREENBOX, trendVector);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      WritePostOverFile(dataBOX, POSTBOX, dataVector, trendVector, legend);

    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrame(residueBOX, SCREENBOX);
    ScreenPlot(residueBOX, SCREENBOX, residueVec);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      WritePostFile(residueBOX, POSTBOX, residueVec, legend);

    ClrScr;

    Writeln;
    Writeln('     FIT A POLYNOMIAL TREND');
    Writeln;
    YesOrNo('    Do you wish to replace the data series by the residual series', replace);

    if replace then
      begin {replace}
        dataBOX := residueBOX;
        dataVector := residueVec;
        MakeDataBox(dataBOX, dataVector);
      end; {replace}

    trendEstimated := true;

  end; {FitPolyTrend}

{________________________________________________}

  procedure LowpassComponent (var dataBOX, trendBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec: longVector);

    var
      residtoDat, trendToDat: boolean;
      t, cutOffInt, diffOrder: integer;
      cutOffDegrees, cutOffRadians: real;
      pi, radiansPerDegree, FourierInc: real;
      alpha, beta: fourierVec;
      legend: string;

  begin
    pi := 4 * ArcTan(1.0);
    radiansPerDegree := pi / 180;

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}

    ClrScr;

    Writeln;
    Writeln('    EXTRACT A LOWPASS COMPONENT');
    Writeln;
    Writeln('    Before  extracting the low-frequency component, you may need to reduce');
    Writeln('    the  data to stationarity via, at  most, two differencing  operations.');
    Writeln('    If you have already extracted the trend by a polynomial regression, or');
    Writeln('    if this is pseudo-random data, then differencing is not required.');
    Writeln;
    Writeln('    Specify the number of differencing operations: 0/1/2.');
    GetBoundedInteger(0, 2, diffOrder); {diffOrder is LOCAL}

    TransferThedata(diffOrder, dataBOX, trendBOX, dataVector, trendVector);

{The TransferThedata procedure serves to transfer the}
{data into the trendVec. It also takes differences.}

    Writeln('    The low-frequency comprises a set of sinusoidal elements of');
    Writeln('    frequencies that range  from zero  to the Fourier frequency');
    Writeln('    nearest to a specified cut-off point.');
    Writeln;

    Writeln('    Specify the cut-off frequency in degrees.');
    ReadANumber(cutOffDegrees);

    fourierInc := 2 * pi / (dataBOX.NOPUNCT);
    cutOffRadians := cutOffDegrees * radiansPerDegree;
    cutOffInt := Round(cutOffRadians / fourierInc);

    FourierCoefficients(trendBOX, trendVector, fourierInc, alpha, beta);
    beta[0] := 0.0;

{Ncap := dataBOX.NOPUNCT div 2;}
    FourierSynthesis(trendBOX, cutOffInt, trendVector, fourierInc, alpha, beta);

    if (diffOrder > 0) then
      DataCumulation(dataBOX, trendBOX, diffOrder, dataVector, trendVector);

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(dataBOX, SCREENBOX);
    ScreenPlot(dataBOX, SCREENBOX, dataVector);
    SetColor(red);
    SetLineStyle(0, 0, ThickWidth);
    ScreenPlot(dataBOX, SCREENBOX, trendVector);
    SetColor(yellow);
    SetLineStyle(0, 0, NormWidth);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict = true then
      WritePostOverFile(dataBOX, POSTBOX, dataVector, trendVector, legend);

    residueBOX := dataBOX;
    with residueBOX do
      begin {residueBOX}
        yMean := 0.0;
        yMax := 0.0;
        yMin := 0.0;
        for t := 0 to dataBOX.NOPUNCT - 1 do
          begin
            residueVec[t] := dataVector[t] - trendVector[t];
            yMax := Rmax(yMax, residueVec[t]);
            yMin := Rmin(yMin, residueVec[t]);
            yMean := yMean + residueVec[t];
          end;
        yMean := yMean / dataBOX.NOPUNCT;
      end; {residueBOX}

    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrame(residueBOX, SCREENBOX);
    ScreenPlot(residueBOX, SCREENBOX, residueVec);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict = true then
      WritePostFile(residueBOX, POSTBOX, residueVec, legend);

    trendEstimated := true;

    ClrScr;

    Writeln;
    Writeln('    EXTRACT A LOWPASS COMPONENT');
    Writeln;
    YesOrNo('    Do you wish to replace the data series by the residual series', residtoDat);
    Writeln;
    if not residtoDat then
      YesOrNo('    Do you wish to replace the data vector by the low-frequency component', trendToDat);

    if (residtoDat or trendToDat) then
      begin {replace}
        for t := 0 to residueBOX.NOPUNCT - 1 do
          begin {t}
            if residtoDat then
              begin
                dataVector[t] := residueVec[t];
                dataBOX := residueBOX;
              end
            else
              dataVector[t] := trendVector[t];
            dataBOX := trendBOX;
          end; {t}

      end; {replace}

    trendEstimated := true;
  end;{LowpassComponent}

{________________________________________________}

end. {DATA: Unit 10 of CONGRESS.PAS}

