unit ORGANISE; {Unit 11 of CONGRESS.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, DATA, ACQUIRE, AUTOREG, ROOTS, SPECTRAL, SCREEN, POST, Dos, Crt, Graph;


  procedure GetTheData (var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure GetPseudoData (var ARMAGen: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure TrendExtract (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBOX, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec: longVector);

  procedure PlotThePeriodogram (var ARMAGen, ARMAEst: ARMARecord;
                  var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure PlotTheSpectrum (var ARMAGen, ARMAEst: ARMARecord;
                  var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure AREstimation (var ARMAGen, ARMAEst: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure SaveSpecifiedModel (var ARMAGen: ARMARecord);

  procedure SaveEstimatedModel (var ARMAEst: ARMARecord);

  procedure SaveAndExit (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var ARMAGen, ARMAEst: ARMARecord);

implementation

 {________________________________________________}

  procedure GetTheData (var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

    var
      theMenuItem: integer;
  begin
    repeat
      ClrScr;
      Writeln;
      Writeln('    GET THE DATA');
      Writeln;
      Writeln('         1. Read the Data');
      Writeln;
      Writeln('         2. Generate Ordinary ARMA Data');
      Writeln;
      Writeln('         3. Generate Band-Limited ARMA Data');
      Writeln;
      Writeln('         4. Plot the Data');
      Writeln;
      Writeln('         5. Take Logs of the Data');
      Writeln;
      Writeln('         6. Subsample the Data');
      Writeln;
      Writeln('         7. RETURN to the Main Menu');
      Writeln;
      Writeln('                      Select a number and press <RETURN>');

      GetBoundedInteger(1, 7, theMenuItem);

      case theMenuItem of
        1: 
          ReadTheData(dataBOX, dataVector); {in UTILS}
        2: 
          GetARMAData(ARMAGen, dataBOX, dataVector); {in DATA}
        3: 
          GetBandLimitedData(ARMAGen, dataBOX, dataVector); {in DATA}
        4: 
          PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in DATA}
        5: 
          LogOfVector(dataBOX, dataVector); {in DATA}
        6: 
          SubsampleTheData(dataBOX, dataVector); {in DATA}
      end;{case}
    until theMenuItem = 7;

  end;{GetTheData}

{________________________________________________}

  procedure GetPseudoData (var ARMAGen: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      bandLimited: boolean;

  begin
    ClrScr;

    Writeln;
    Writeln('    GENERATE THE ARMA DATA');
    Writeln;

    YesOrNo('    Do you wish to generate band-Limited data', bandLimited);
    if bandLimited then
      GetBandLimitedData(ARMAGen, dataBOX, dataVector) {in DATA}
    else
      begin {not bandLimited}

        if ARMAModel then
          begin
            SysBeep(3);
            YesOrNo('    You have already specified the model. Do you wish to retain it', ARMAModel);
          end;

        if (ARMAModel = false) then
          RationalAcquisition(ARMAGen, stability);

        ARMAGen.varepsilon := 1.0;
        Writeln('    What is size T of the data sample you wish to generate?');
        GetBoundedInteger(16, 500, dataBOX.NOPUNCT);
        stability := true;
        GenerateData(ARMAGen, dataBOX, dataVector);
      end; {not bandLimited}

    thereIsData := true;
    pseudoData := true;

  end; {GetPseudoData}

{_________________________________________________}

  procedure TrendExtract (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBOX, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec: longVector);

    var
      theMenuItem: integer;

  begin

    if thereIsData = false then
      GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}
      end;

    repeat
      ClrScr;

      Writeln;
      Writeln('    LOWPASS FILTERING AND TREND EXTRACTION');
      Writeln;
      Writeln('         1. Extract a Polynomial Trend');
      Writeln;
      Writeln('         2. Apply a Lowpass Trigonometrical Filter');
      Writeln;
      Writeln('         3. RETURN to the Main Menu');
      Writeln;
      Writeln('                      Select a number and press RETURN');

      GetBoundedInteger(1, 3, theMenuItem);

      case theMenuItem of
        1:  {in DATA}
          FitPolyTrend(dataBOX, trendBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec);
        2: {in DATA}
          LowpassComponent(dataBOX, trendBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec);
      end;

    until theMenuItem = 3;

  end;{TrendExtract}

{________________________________________________}

  procedure PlotThePeriodogram (var ARMAGen, ARMAEst: ARMARecord;
                  var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

    var
      menuChoice: char;
      periodoVec, ARMASpecVec: longVector;
      plotBOX, periodoBOX, spectrumBOX: rBOX;
      postFileName, legend: string;
      F: text;

    procedure WriteOptionD;
    begin
      Writeln('        (d) Plot the periodogram of the data');
    end;
    procedure WriteOptionG;
    begin
      Writeln('        (g) Plot the periodogram of the data and the parametric spectrum ');
      Writeln('             of the process generating the data ');
    end;
    procedure WriteOptionE;
    begin
      Writeln('        (e) Plot the periodogram of the data and the parametric spectrum ');
      Writeln('             of the model estimated from the data');
    end;

  begin {PlotThePeridoogram}

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    if thereIsData = false then
      GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}

    menuChoice := 'x';
    if empiricData then
      legend := 'The peridodogram of an empirical data sequence';
    if pseudoData then
      legend := 'The periodogram of the pseudo random data';

    ClrScr;

    Writeln;
    Writeln('    `PLOT THE PERIODOGRAM');
    Writeln;
    Writeln('    This section gives a choice of two methods of calculcatiing the Fourier');
    Writeln('    transform of the data, which  are a slow  method that preserves  all of');
    Writeln('    the data and a method tha employs a mixed radix fast Fourier transform.');
    Writeln('    The latter requires that  the sample  size should  be composite number,');
    Writeln('    comprising no less than three prime factors, none  of which must exceed');
    Writeln('    23. To acheive this, it may be necessary to  discard some sample points.');
    Writeln;
    Writeln(' ' : 66, '<RETURN>');
    repeat
      Writeln;
      Writeln('    1. The Slow Peridogram');
      Writeln;
      Writeln('    2. The Fast Peridogram');
      Writeln;
      Writeln('                      Select a number and press <RETURN>');
      Readln(slowOrFast)
    until slowOrFast in [1, 2];

    if empiricdata and ARestimates then
      repeat
        ClrScr;

        Writeln;
        Writeln('    PLOT THE PERIODOGRAM');
        Writeln;
        WriteOptionD; {Periodogam Alone}
        Writeln;
        WriteOptionE; {Peridodogam and Estimated Model}
        Writeln;
        Writeln('    Type <d> or <e> and press <RETURN>');
        ReadAChar(menuChoice);
        if menuChoice in ['e', 'E'] then
          legend := Concat(legend, ' superimposed on the parametric spectrum of an estimated model');
        if not (menuChoice in ['d', 'D', 'e', 'E']) then
          Sysbeep(3);
      until menuChoice in ['d', 'D', 'e', 'E'];

    if pseudoData and (not ARestimates) and (not subsampledData) then
      repeat
        ClrScr;

        Writeln;
        Writeln('    PLOT THE PERIODOGRAM');
        Writeln;
        WriteOptionD; {Periodogam Alone}
        Writeln;
        WriteOptionG; {Periodogam and Generating Process}
        Writeln;
        Writeln('    Type <d>  or <g> and press <RETURN>');
        ReadAChar(menuChoice);
        if menuChoice in ['G', 'g'] then
          legend := Concat(legend, ' superimposed on the spectrum of the process generating the data');
        if not (menuChoice in ['d', 'D', 'g', 'G']) then
          Sysbeep(3);
      until menuChoice in ['d', 'D', 'g', 'G'];

    if pseudoData and ARestimates then
      repeat
        ClrScr;

        Writeln;
        Writeln('    PLOT THE PERIODOGRAM');
        Writeln;
        WriteOptionD; {Periodogam Alone}
        Writeln;
        WriteOptionG; {Periodogam and Generating Process}
        Writeln;
        WriteOptionE;  {Peridodogam and Estimated Model}
        Writeln;
        Writeln('    Type <d> or <e> or <g> and press <RETURN>');
        ReadAChar(menuChoice);
        if menuChoice in ['e', 'E'] then
          legend := Concat(legend, ' superimposed on the parametric spectrum of an estimated model ');
        if menuChoice in ['G', 'g'] then
          legend := Concat(legend, ' superimposed on the spectrum of the process generating the data');
        if not (menuChoice in ['d', 'D', 'e', 'E', 'g', 'G']) then
          Sysbeep(3);
      until menuChoice in ['d', 'D', 'e', 'E', 'g', 'G'];

    ClrScr;
    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    if (slowOrFast = Slow) then
      SlowPeriodogram(dataBOX, periodoBOX, dataVector, periodoVec)
    else
      FastPeriodogram(dataBOX, periodoBOX, dataVector, periodoVec);

    plotBOX := periodoBOX;

    if menuChoice in ['G', 'g'] then
      begin {Spectrum of Generating Process}
{Getting the spectral vectors}
        if ARMAGen.bandLimits and (not subsampledData) then
          begin
            MakeScreenBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, spectrumBOX);

            plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
            ScreenFrameLite(plotBOX, SCREENBOX);

            SetColor(red);
            SetLineStyle(0, 0, ThickWidth);
            ScreenBandPlot(ARMAGen, plotBOX, SCREENBOX, ARMASpecVec);
            SetColor(yellow);
            SetLineStyle(0, 0, NormWidth);
            LimitLine(SCREENBOX, ARMAGen);
          end
        else
          begin
            MakeScreenGenSpec(ARMAGen, SCREENBOX, ARMASpecVec, spectrumBOX);

            plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
            ScreenFrameLite(plotBOX, SCREENBOX);

            SetColor(red);
            SetLineStyle(0, 0, ThickWidth);
            ScreenPlotMod(plotBOX, SCREENBOX, ARMASpecVec);
            SetColor(yellow);
            SetLineStyle(0, 0, NormWidth);
          end;
      end; {Spectrum of Generating Process}

    if menuChoice in ['E', 'e'] then
      begin {Estimated Spectrum}
        MakeAREstimSpec(ARMAEst, SCREENBOX, ARMASpecVec, spectrumBOX);

        plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
        ScreenFrameLite(plotBOX, SCREENBOX);

        SetColor(red);
        SetLineStyle(0, 0, ThickWidth);
        plotBOX.NOPUNCT := spectrumBOX.NOPUNCT;
        ScreenPlotMod(plotBOX, SCREENBOX, ARMASpecVec);
        SetColor(yellow);
        SetLineStyle(0, 0, NormWidth);

      end; {Estimated Spectrum}

    if menuChoice in ['x', 'd', 'D'] then
      ScreenFrameLite(plotBOX, SCREENBOX);

    plotBOX.NOPUNCT := periodoBOX.NOPUNCT;
    ScreenPlot(plotBOX, SCREENBOX, periodoVec);
    ZeroPiAxis(SCREENBOX);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      begin {Make a PostScript File}
        Writeln;
        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then

          begin {postFileName}
            Assign(F, postFileName);
            ReWrite(F);
            PostHeader(F, POSTBOX, TeXFormat);

            if menuChoice in ['x', 'd', 'D'] then
              begin
                PostFrameLite(F, periodoBOX, POSTBOX);
                PostZeroPiAxis(F, POSTBOX);
                PostPlot(F, plotBOX, POSTBOX, periodoVec);
              end;

            if menuChoice in ['G', 'g'] then
              begin {Spectrum of Generating Process}
                if ARMAGen.bandLimits and (not subsampledData) then
                  begin {Special Provisions for POST}
                    MakePostBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, spectrumBOX);

                    plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
                    PostFrameLite(F, plotBOX, POSTBOX);
                    PostZeroPiAxis(F, POSTBOX);

                    plotBOX.NOPUNCT := spectrumBOX.NOPUNCT;
                    PostBandPlot(F, ARMAGen, plotBOX, POSTBOX, ARMASpecVec);
                    PostLimitLine(F, POSTBOX, ARMAGen);

                    plotBOX.NOPUNCT := periodoBOX.NOPUNCT;
                    PostPlot(F, plotBOX, POSTBOX, periodoVec);
                  end
                else
                  begin
{ARMASpecVec <-- MakeScreenGenSpec (MakeDataGenSpec): both SCREEN and POST}

                    plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
                    PostFrameLite(F, plotBOX, POSTBOX);
                    PostZeroPiAxis(F, POSTBOX);

                    plotBOX.NOPUNCT := spectrumBOX.NOPUNCT;
                    PostPlotMod(F, plotBOX, POSTBOX, ARMASpecVec);

                    plotBOX.NOPUNCT := periodoBOX.NOPUNCT;
                    PostPlot(F, plotBOX, POSTBOX, periodoVec);
                  end;
              end; {Spectrum of Generating Process}

            if menuChoice in ['E', 'e'] then
              begin {Estimated Spectrum}
{ARMASpecVec <--MakeAREstimSpec: both SCREEN and POST}

                plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
                PostFrameLite(F, plotBOX, POSTBOX);
                PostZeroPiAxis(F, POSTBOX);

                plotBOX.NOPUNCT := spectrumBOX.NOPUNCT;
                PostPlotMod(F, plotBOX, POSTBOX, ARMASpecVec);

                plotBOX.NOPUNCT := periodoBOX.NOPUNCT;
                PostPlot(F, plotBOX, POSTBOX, periodoVec);
              end; {Estimated Spectrum}

            PostFooter(F, POSTBOX, legend, TexFormat);
            Close(F);
          end; {postFileName}
      end; {Make a PostScript File}

  end; {PlotThePeridogram}

{________________________________________________}

  procedure PlotTheSpectrum (var ARMAGen, ARMAEst: ARMARecord;
                  var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

    var
      ARMASpecVec, AREstSpecVec, periodoVec: longVector;
      generBOX, estimBOX, spectraBOX, periodoBOX: rBOX;
      legend, postFileName: string;
      F: text;

  begin

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    if (ARMAModel) and (ARestimates = false) then
      begin {(M1, E0)}
        ClrScr;

        Writeln;
        Writeln('    PLOT THE SPECTRUM');
        Writeln;
        Writeln('    You have specified an ARMA model that may be used to generate');
        Writeln('    pseudo-random data. The spectrum of the  model is displayed.');
        Writeln;
        Writeln(' ' : 57, '<RETURN>');
        Readln;
        legend := 'The parametric spectrum of a model specified by the user';
      end; {(M1, E0)}

    if ((ARMAModel = false) and (ARestimates = false)) then
      begin {(M0, E0) --> (M1, E0)}
        ClrScr;

        Writeln;
        Writeln('    PLOT THE SPECTRUM');
        Writeln;
        Writeln('    You have not specified an ARMA model and you have not estimated');
        Writeln('    any parameters. You are asked to specify an ARMA model of which');
        Writeln('    the spectral density function will be displayed.');
        Writeln;
        Writeln(' ' : 59, '<RETURN>');
        Readln;
        legend := 'The parametric spectrum of a model specified by the user';
        RationalAcquisition(ARMAGen, stability);
        ARMAGen.varepsilon := 1.0;
        ARMAModel := true;
      end; {(M0, E0) --> (M1, E0)}

    if ARMAModel and not ARestimates then
      begin {M1, E0}
{Getting the spectral vectors}
        if ARMAGen.bandLimits then
          MakeScreenBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX)
        else
          MakeScreenGenSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX);

{Plotting}
        GetGraphics(graphicsOn, SCREENBOX, currentMode);

        ScreenFrameLite(generBOX, SCREENBOX);
        ZeroPiAxis(SCREENBOX);

        if ARMAGen.bandLimits then
          begin
            ScreenBandPlot(ARMAGen, generBOX, SCREENBOX, ARMASpecVec);
            SetColor(yellow);
            SetLineStyle(0, 0, NormWidth);
            LimitLine(SCREENBOX, ARMAGen);
          end
        else
          ScreenPlotMod(generBOX, SCREENBOX, ARMASpecVec);
        legend := 'The parametric spectrum of a model specified by the user';
        ScreenCaption(legend);

{do the PostScript}
        if PSPict then
          begin {Make a PostScript File}
            CallForReturn;
            Readln;
            CloseGraph;
            RestoreCrtMode;
            ClrScr;

            Writeln;
            Write('    Name the PostScript File = ');
            Readln(postFileName);
            if postFileName <> '' then
              begin {postFileName}
                Assign(F, postFileName);
                Rewrite(F);
                PostHeader(F, POSTBOX, TeXFormat);
                PostFrameLite(F, generBOX, POSTBOX);
                PostZeroPiAxis(F, POSTBOX);
                if ARMAGen.bandLimits then
                  begin
                    MakePostBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX);
                    PostBandPlot(F, ARMAGen, generBOX, POSTBOX, ARMASpecVec);
                    PostLimitLine(F, POSTBOX, ARMAGen);
                  end
                else
                  PostPlotMod(F, generBOX, POSTBOX, ARMASpecVec);
                PostFooter(F, POSTBOX, legend, TexFormat);
                Close(F);
              end; {postFileName}
          end; {Make a PostScript File}

      end; {M1, E0}

    if ARMAModel and ARestimates and pseudoData then
      begin {M1, E1}
        ClrScr;

        Writeln;
        Writeln('    PLOT THE SPECTRUM');
        Writeln;
        Writeln('    You  have estimated  an AR model from  pseudo-random  data.');
        Writeln('    The parametric spectrum of the estimated model is displayed');
        Writeln('    together with that of the process generating the data.');
        Writeln;
        Writeln(' ' : 55, '<RETURN>');
        Readln;

{Getting the spectral vectors}
        MakeAREstimSpec(ARMAEst, SCREENBOX, AREstSpecVec, estimBOX);
        if ARMAGen.bandLimits and (not subSampledData) then
          MakeScreenBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX)
        else
          MakeScreenGenSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX);

{Determining the framework}
        if generBOX.yMax > estimBOX.yMax then
          spectraBOX := generBOX
        else
          spectraBOX := estimBOX;

        {Plotting}
        GetGraphics(graphicsOn, SCREENBOX, currentMode);

        ScreenFrameLite(spectraBOX, SCREENBOX);
        ZeroPiAxis(SCREENBOX);

        if ARMAGen.bandLimits and (not subSampledData) then
          ScreenBandPlot(ARMAGen, spectraBOX, SCREENBOX, ARMASpecVec)
        else
          ScreenPlotMod(spectraBOX, SCREENBOX, ARMASpecVec);

        SetColor(red);
        SetLineStyle(0, 0, ThickWidth);
        ScreenPlotMod(spectraBOX, SCREENBOX, AREstSpecVec);
        SEtColor(yellow);
        SetLineStyle(0, 0, NormWidth);

        legend := 'The specta of the estimated model (in red) and of the process generating the data';
        ScreenCaption(legend);

{Do the PostScript}
        if PSPict then
          begin {Make a PostScript File}
            CallForReturn;
            Readln;
            CloseGraph;
            RestoreCrtMode;
            ClrScr;

            Writeln;
            Write('    Name the PostScript File = ');
            Readln(postFileName);
            if postFileName <> '' then
              begin {postFileName}
                Assign(F, postFileName);
                Rewrite(F);
                PostHeader(F, POSTBOX, TeXFormat);
                PostFrameLite(F, spectraBOX, POSTBOX);
                PostZeroPiAxis(F, POSTBOX);
                if ARMAGen.bandLimits and (not subSampledData) then
                  begin
                    MakePostBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, spectraBOX);
                    PostBandPlot(F, ARMAGen, spectraBOX, POSTBOX, ARMASpecVec);
                  end
                else
                  PostPlotMod(F, spectraBOX, POSTBOX, ARMASpecVec);
                PostPlotMod(F, spectraBOX, POSTBOX, AREstSpecVec);
                PostFooter(F, POSTBOX, legend, TexFormat);
                Close(F);
              end; {postFileName}
          end; {Make a PostScript File}
      end; {M1, E1}

    if not ARMAModel and ARestimates then
      begin {M0, E1}
        ClrScr;

        Writeln;
        Writeln('    PLOT THE SPECTRUM'); {no Model Estimtes M0, E1}
        Writeln;
        Writeln('    You have  estimated an AR model from empirical data. The');
        Writeln('    parametric spectrum of the estimated  model is displayed');
        Writeln('    together with the periodogram of the data.');
        Writeln;
        Writeln(' ' : 52, '<RETURN>');
        Readln;

{What about the scale:  should this be passed to peridodogram?}
{peridoVec --> ARMASpecVec: to save memory}
        if (slowOrFast = Slow) then
          SlowPeriodogram(dataBOX, periodoBOX, dataVector, periodoVec)
        else
          FastPeriodogram(dataBOX, periodoBOX, dataVector, periodoVec);

        MakeAREstimSpec(ARMAEst, SCREENBOX, AREstSpecVec, estimBOX);
        if estimBOX.yMax > periodoBOX.yMax then
          periodoBOX.yMax := estimBOX.yMax
        else
          estimBOX.yMax := periodoBOX.yMax;

        GetGraphics(graphicsOn, SCREENBOX, currentMode);

        ScreenFrameLite(periodoBOX, SCREENBOX);
        estimBOX.yUB := periodoBOX.yUB;
        estimBOX.yLB := 0.0;

        ZeroPiAxis(SCREENBOX);
        ScreenPlot(periodoBOX, SCREENBOX, periodoVec);
        SetColor(red);
        SetLineStyle(0, 0, ThickWidth);
        ScreenPlotMod(estimBOX, SCREENBOX, AREstSpecVec);
        SetColor(yellow);
        SetLineStyle(0, 0, NormWidth);

        legend := 'The spectum of the estimated model and the periododogram of the empirical data.';
        ScreenCaption(legend);

{Do the PostScript}
        if PSPict then
          begin {Make a PostScript File}
            CallForReturn;
            Readln;
            CloseGraph;
            RestoreCrtMode;
            ClrScr;

            Writeln;
            Writeln('Name the PostScript File');
            Readln(postFileName);
            if postFileName <> '' then
              begin {postFileName}
                Assign(F, postFileName);
                ReWrite(F);
                PostHeader(F, POSTBOX, TeXFormat);
                PostFrameLite(F, periodoBOX, POSTBOX);
                PostZeroPiAxis(F, POSTBOX);
                PostPlot(F, periodoBOX, POSTBOX, periodoVec);
                PostPlotMod(F, periodoBOX, POSTBOX, AREstSpecVec);
                PostFooter(F, POSTBOX, legend, TexFormat);
                Close(F);
              end; {postFileName}
          end; {Make a PostScript File}
      end; {M0, E1}

    if not PSPict then
      begin {clear the Graphics}
        CallForReturn;
        Readln;
        CloseGraph;
        RestoreCrtMode;
        ClrScr
      end; {clear the Graphics}

  end; {PlotTheSpectrum}

{_________________________________________________}

  procedure AREstimation (var ARMAGen, ARMAEst: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);
    var
      aCovar, gamma: vector;
      lags, t, i: integer;
      correctionFactor: real;
      cirCovar: longVector;
      theChoice: char;
      abortit, zerosQ, MAComponent: boolean;

  begin

    if thereIsData = false then
      GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}

    if thereIsData then
      begin {thereIsData}

        abortit := false;

        if (trendEstimated = false) then
          TestForStationarity(dataBOX, dataVector, abortit);
        if (abortit = false) then

          begin {abortit = false}
            ClrScr;

            ARMAEst.p := 0;
            ARMAEst.q := 0;

            Writeln;
            Writeln('    AUTOREGRESSIVE (AND MOVING-AVERAGE) ESTIMATION');
            Writeln;
            Writeln;
            Writeln('    What is the order of the estimated autoregressive component?');
            GetBoundedInteger(0, 15, ARMAEst.p);
            lags := 2 * ARMAEst.p;
            ARMAEst.mu[0] := 1.0;
            Writeln;
            YesOrNo('    Do you wish to impose zeros at the zero or Nyquist frequency', ZerosQ);
            if zerosQ then
              begin {impose zeros}
                ImposeZeros(ARMAEst);
                PreSpecifiedZeros := true;
              end; {impose zeros}

            if (ZerosQ = false) then
              begin {Unrestricted MA}
                Writeln('    You may wish to have a freely estimated MA component? The order');
                Writeln('    of the  MA component must  not exceed that of  the AR component');
                Writeln;
                YesOrNo('    Do you wish to estimate an unrestricted MA component', MAComponent);
                if MAComponent then
                  begin
                    Writeln('    What is the order of the MA component?');
                    GetBoundedInteger(0, ARMAEst.p, ARMAEst.q);
                  end;
              end; {Unrestricted MA}

            Writeln;
            repeat
              ClrScr;

              Writeln;
              Writeln('    CALCULATE THE AUTOCOVARIANCES');
              Writeln;
              Writeln('    Type (c) for circular autocovariances (a) for ordinary autocovariances');
              ReadAChar(theChoice);
            until theChoice in ['c', 'C', 'a', 'A'];

            lags := ARMAEst.p + ARMAEst.q;

            if theChoice in ['a', 'A'] then
              AutoCovariances(dataBOX.NOPUNCT, lags, dataVector, aCovar)
            else
              begin {Circular Autocovariances}
                CircularACovars(dataBOX.NOPUNCT, dataBOX.NOPUNCT, dataVector, cirCovar);
                for t := 0 to lags do
                  aCovar[t] := cirCovar[t];
              end; {Circular Autocovariances}

            ARMAparameters(ARMAEst, aCovar);

            PoleZeroEstimates(ARMAEst);

            ClrScr;

            Writeln;
            Writeln('    THE MOVING-AVERAGE PARAMETERS');
            Writeln;
            for i := 0 to ARMAEst.q do
              Writeln('    mu[', i : 1, '] = ', ARMAEst.mu[i] : 6 : 4);

            ARMACovariances(ARMAEst, gamma);

            if PreSpecifiedZeros then
              begin
                correctionFactor := ACovar[0] / ARMAEst.yVar;
                ARMAEst.varEpsilon := ARMAEst.varEpsilon * correctionFactor;
              end;

            Writeln;
{this where we need to develop the correction factor}

            Writeln('    THE AUTOREGRESSIVE ESTIMATES');
            Writeln;
            for i := 0 to ARMAEst.p do
              Writeln('    alpha[', i : 1, '] = ', ARMAEst.alpha[i] : 6 : 4);

            Writeln;
            Writeln(' ' : 60, '<RETURN>');
            Readln;

          end; {thereIsData}

        ARestimates := true;
        PreSpecifiedZeros := false;
      end; {abortit = false}

  end; {AREstimation}

{________________________________________________}

  procedure SaveSpecifiedModel (var ARMAGen: ARMARecord);

    var
      j: integer;
      F: text;

  begin
    if ARMAModel = false then
      begin
        ClrScr;

        Writeln;
        Writeln('    SAVE THE SPECIFIED MODEL');
        Writeln;
        Writeln('    No Model has been specified');
        if ARestimates then
          Writeln('    A model has been estimated from the data');
        Writeln(' ' : 60, '<RETURN>');
        Readln;
      end
    else
      begin {ARMAModel}
        Assign(F, 'SpecifiedModel.txt');
        ReWrite(F);

        Writeln;
        Writeln(F, '    THE PARAMETERS OF THE SPECIFIED MODEL');
        Writeln(F);
        Writeln(F, '    The Autoregressive Parameters of the Specified Model');
        Writeln(F);
        for j := 0 to ARMAGen.p do
          Writeln(F, '    alpha[', j : 1, '] = ', ARMAGen.alpha[j] : 6 : 4);
        Writeln(F);
        for j := 1 to ARMAGen.p do
          Writeln(F, '    poles[', j : 1, '].re = ', ARMAGen.poles[j].re : 5 : 5, '   poles[', j : 1, '].im = ', ARMAGen.poles[j].im : 5 : 5);
        Writeln(F);
        for j := 1 to ARMAGen.p do
          begin
            Write(F, '    root(', j : 1, ').modu = ', ARMAGen.Polarpoles[j].modu : 8 : 4);
            Writeln(F, '    root(', j : 1, ').argu = ', ARMAGen.Polarpoles[j].argu : 8 : 4);
          end;
        Writeln(F);

        if (ARMAGen.q > 0) then
          begin {MA parameters}
            Writeln(F, '    The Moving-Average Parameters of the Specified Model');
            Writeln(F);
            for j := 0 to ARMAGen.q do
              Writeln(F, '    mu[', j : 1, '] = ', ARMAGen.mu[j] : 6 : 4);
            Writeln(F);
            for j := 1 to ARMAGen.q do
              begin
                Write(F, '    zeros[', j : 1, '].re = ', ARMAGen.zeros[j].re : 5 : 5);
                Writeln(F, '    zeros[', j : 1, '].im = ', ARMAGen.zeros[j].im : 5 : 5);
              end;
            Writeln(F);
            for j := 1 to ARMAGen.q do
              begin
                Write(F, '    root(', j : 1, ').modu = ', ARMAGen.Polarzeros[j].modu : 8 : 4);
                Writeln(F, '    root(', j : 1, ').argu = ', ARMAGen.Polarzeros[j].argu : 8 : 4);
              end;
          end; {MA Parameters}

        Close(F);

      end; {ARMAModel}

  end; {SaveSpecifiedModel}

{_________________________________________________}

  procedure SaveEstimatedModel (var ARMAEst: ARMARecord);

    var
      j: integer;
      F: text;

  begin
    if (ARestimates = false) then
      begin
        ClrScr;

        Writeln;
        Writeln('    SAVE THE ESTIMATED MODEL');
        Writeln;
        Writeln('    No Model has been estimated');
        Writeln(' ' : 60, '<RETURN>');
        Readln;
      end
    else
      begin {ARMAEstimates}
        Assign(F, 'EstimatedModel.txt');
        ReWrite(F);

        Writeln(F);
        Writeln(F, '    THE PARAMETERS OF THE ESTIMATED MODEL');
        Writeln(F);
        Writeln(F, '    The Autoregressive Parameters of the Estimated Model');
        Writeln(F);
        for j := 0 to ARMAEst.p do
          Writeln(F, '    alpha[', j : 1, '] = ', ARMAEst.alpha[j] : 6 : 4);
        Writeln(F);
        for j := 1 to ARMAEst.p do
          begin
            Write(F, '    poles[', j : 1, '].re = ', ARMAEst.poles[j].re : 5 : 5);
            Writeln(F, '    poles[', j : 1, '].im = ', ARMAEst.poles[j].im : 5 : 5);
          end;
        Writeln(F);
        for j := 1 to ARMAEst.p do
          begin
            Write(F, '    root(', j : 1, ').modu = ', ARMAEst.Polarpoles[j].modu : 8 : 4);
            Writeln(F, '    root(', j : 1, ').argu = ', ARMAEst.Polarpoles[j].argu : 8 : 4);
          end;
        Writeln(F);

        if (ARMAEst.q > 0) then
          begin {MA parameters}
            Writeln(F, '    The Moving-Average Parameters of the Estimated Model');
            Writeln(F);
            for j := 0 to ARMAEst.q do
              Writeln(F, '    mu[', j : 1, '] = ', ARMAEst.mu[j] : 6 : 4);
            Writeln(F);
            for j := 1 to ARMAEst.q do
              begin
                Write(F, '    zeros[', j : 1, '].re = ', ARMAEst.zeros[j].re : 5 : 5);
                Writeln(F, '    zeros[', j : 1, '].im = ', ARMAEst.zeros[j].im : 5 : 5);
              end;
            Writeln(F);
            for j := 1 to ARMAEst.p do
              begin
                Write(F, '    root(', j : 1, ').modu = ', ARMAEst.Polarzeros[j].modu : 8 : 4);
                Writeln(F, '    root(', j : 1, ').argu = ', ARMAEst.Polarzeros[j].argu : 8 : 4);
              end;
            Writeln(F);
          end; {MA Parameters}

        Close(F);

      end; {ARMAEstimates}

  end; {SaveEstimatedModel}

{________________________________________________}

  procedure SaveAndExit (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var ARMAGen, ARMAEst: ARMARecord);

    var
      theMenuItem: integer;

  begin
    repeat
      ClrScr;

      Writeln;
      Writeln('    SAVE AND EXIT');
      Writeln;
      Writeln('      1. Save the Data');
      Writeln;
      Writeln('      2. Save the Specified ARMA Model');
      Writeln;
      Writeln('      3. Save the Estimated ARMA Model');
      Writeln;
      Writeln('      4. RETURN to the Main Menu');
      Writeln;
      Writeln('      5. EXIT the Program');
      Writeln;
      Writeln('                      Select a number and press RETURN');
      GetBoundedInteger(1, 5, theMenuItem);

      case theMenuItem of
        1: {in UTILS}
          WriteData(dataBOX, dataVector);
        2: {in ORGANISE}
          SaveSpecifiedModel(ARMAGen);
        3: {in ORGANISE}
          SaveEstimatedModel(ARMAEst);
        4: 
          ;{RETURN to the Main Menu}
        5: 
          allsDone := true; {EXIT the Program}
      end;{case}

    until theMenuItem in [1..5];

  end;{SaveAndExit}

{________________________________________________}

end. {ORGANISE: Unit 11 of CONGRESS.PAS}

