unit UTILS; {Unit 3 of BLIMDOS.PAS}

interface

  uses
    GLOBALS, MATHS, SysUtils, Graph, Dos, Crt;

  procedure DummyProcedure;

  procedure YesOrNo (question: string;
                  var decision: boolean);

  procedure InitialiseGraphics (var SCREENBOX: IBOX;
                  var currentMode: integer);

  procedure GetGraphics (var graphicsOn: boolean;
                  var SCREENBOX: IBOX;
                  var currentMode: integer);

  procedure SetTheText;

  procedure RevertText;

  procedure CallForReturn;

  procedure ScreenCaption (legend: string);

  procedure SysBeep (pause: integer);

  function TickCount: longint;

  function UpCase (character: char): char;

  function DownCase (character: char): char;

  function MenuChar (alpha, omega: char): char;

  procedure PageParameters (var SCREENBOX, POSTBOX: IBOX);

  procedure PageSetUp (var SCREENBOX, POSTBOX: IBOX;
                  var PSPict, TeXFormat: boolean);

  procedure StripLeadingBlanks (var theString: string);

  procedure StripTrailingBlanks (var theString: string);

  procedure ReplaceEveryCaseOf (deletion, replacement: string;
                  var theString: string);

  procedure TallyPoints (range: real;
                  var taly: real);

  function IntegerToString (theInteger: integer): string;

  function TalyToString (theNumber: real): string;

  function IntegerString (theString: string): boolean;

  function DecimalString (theString: string): boolean;

  function StringToInteger (theString: string): longint;

  procedure GetBoundedInteger (lowerBound, upperBound: integer;
                  var theInteger: integer);

  procedure ReadAChar (var theCharacter: char);

  procedure ReadANumber (var theNumber: real);

  procedure ReadBoundedNumber (lowerBound, upperBound: real;
                  var theNumber: real);

  function LineStatus (theLine: string): integer;

  procedure ReadANumericLine (theLine: string;
                  var foundNo: integer;
                  var numberVec: vector);

  procedure ReadDataSafely (var realBOX: rBOX;
                  var dataVector: longVector;
                  maxArray: integer);

  procedure ReadTheData (var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure WriteData (var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure MakeDataBox (var dataBOX: rBOX;
                  var dataVector: longVector);

implementation

   {________________________________________________}

  procedure DummyProcedure;
  begin
  end; {DummyProcedure}

   {________________________________________________}

  procedure YesOrNo (question: string;
                  var decision: boolean);

    var
      contrary: boolean;
      answer: char;

  begin
    repeat
      Writeln(question, ' Y/N?');
      Read(Answer);
      Readln;
      decision := (answer = 'Y') or (answer = 'y');
      contrary := (answer = 'N') or (answer = 'n');
    until (decision <> contrary);
  end; {YesOrNo}

   {__________________________________________________}

  procedure InitialiseGraphics (var SCREENBOX: IBOX;
                  var currentMode: integer);
      {This routine is specific to Turbo Pascal}

    var
      graphDriver, graphMode, CTX, CTY: integer;
      driverPath: string;

  begin
        {DetectGraph(graphDriver, graphMode);}
        {driverPath := 'C:\BP\BGI';}
        {driverPath := '..\PROGFILE\BGI';}
        {include BGI in PROGFILE: the program file}
        {graphDriver := detect;}
        {graphMode := 0;}

    driverPath := ' ';

       {graphDriver := D8bit; }
     {Alternative 1}
       {graphMode := m640x480;}
     {Alternative 1}

    graphDriver := detect;  {Alternative 2}
    DetectGraph(graphDriver, graphMode);   {Alternative 2}

    InitGraph(graphDriver, graphMode, driverPath);

    if GraphResult <> grOK then
      begin
        Writeln('    FATAL ERROR: graphics initialisation error');
        Halt(1);
      end;

    graphicsOn := true;
    SetBkColor(blue);
    SetColor(yellow);
    ClearDevice;
    currentMode := GetGraphMode;

    CTX := GetMaxX div 2;
    CTY := GetMaxY div 2;
    CTY := CTY - 20;

    if noSCREENBOX then
      begin
        SCREENBOX.X0 := SCREENBOX.X0 + CTX;
        SCREENBOX.Y0 := SCREENBOX.Y0 + CTY;
        SCREENBOX.X1 := SCREENBOX.X1 + CTX;
        SCREENBOX.Y1 := SCREENBOX.Y1 + CTY;
        SCREENBOX.X2 := SCREENBOX.X2 + CTX;
        SCREENBOX.Y2 := SCREENBOX.Y2 + CTY;
        SCREENBOX.XC := CTX;
        SCREENBOX.YC := CTY;

        noSCREENBOX := false;
      end;

  end; {InitialiseGraphics}

   {_________________________________________________}

  procedure GetGraphics (var graphicsOn: boolean;
                  var SCREENBOX: IBOX;
                  var currentMode: integer);
      {This routine is specific to Turbo Pascal}

  begin
    if graphicsOn then
      begin
        SetGraphMode(currentMode);
        SetBkColor(blue);
        SetColor(yellow);
        ClearDevice;
      end
    else
      InitialiseGraphics(SCREENBOX, currentMode);
        {graphicsOn := true;}
    graphicsOn := false;
  end; {GetGraphics}

   {_________________________________________________}

  procedure SetTheText;
     {This routine is specific to Turbo Pascal}

  begin
    TextBackGround(blue);
    ClrScr;
  end; {SetTheText}

   {__________________________________________________}

  procedure RevertText;
      {This routine is specific to Turbo Pascal}

  begin
    TextBackGround(black);
    ClrScr;
  end; {RevertText}

   {__________________________________________________}

  procedure CallForReturn;
      {This routine is specific to Turbo Pascal}

    var
      message: string;
      width, XS, YS: integer;
  begin
       {message := 'To Continue, Press <RETURN>';}
    message := 'To Continue, Close the Graph Window and Press <RETURN>';
    width := TextWidth(message);
    XS := (GetMaxX div 2) - (width div 2);
    YS := 10;
    MoveTo(XS, YS);
    OutText(message);
  end; {CallForReturn}

  {_________________________________________________}

  procedure ScreenCaption (legend: string);

    var
      i, q, width, distance, breakPoint, XS, XQ, YS: integer;
      firstString, secondString: string;

  begin
    XS := GetMaxX div 2;
    YS := (GetMaxY) - 30;

    width := Length(legend);

    if width > 65 then
      begin {Breakit }
        distance := width;
        for i := 1 to width do
          begin {i}
            if legend[i] = ' ' then
              begin {if legend}
                q := Abs(width div 2 - i);
                if distance > q then
                  begin {if distance}
                    breakPoint := i;
                    distance := q;
                  end; {if distance}
              end; {if legend}
          end; {i}

        firstString := Copy(legend, 1, breakPoint);
        secondString := Copy(legend, breakPoint + 1, width);
        width := TextWidth(firstString);
        XQ := XS - (width div 2);
        YS := YS - 35;
        YS := SCREENBOX.Y2 + 60; {Alternative}
        MoveTo(XQ, YS);
        OutText(firstString);
        width := TextWidth(secondString);
        XQ := XS - (width div 2);
        YS := YS + 15;
        MoveTo(XQ, YS);
        OutText(secondString);
      end {Breakit}
    else
      begin
        width := TextWidth(legend);
        XS := XS - (width div 2);
        YS := YS - 30;
        YS := SCREENBOX.Y2 + 60; {Alternative}
        MoveTo(XS, YS);
        OutText(legend);
      end;

  end; {ScreenCaption}

   {_________________________________________________}

  procedure SysBeep (pause: integer);

  begin
    Sound(220);
    Delay(pause * 30);
    NoSound;
  end; {SysBeep}

   {________________________________________________}

  function TickCount: longint;
    var
      hour, minute, second, tick: word;
  begin
    GetTime(hour, minute, second, tick);
    TickCount := minute * 3600 + second * 60 + (tick * 60) div 100;
  end; {TickCount}

  {__________________________________________________}


  function UpCase (character: char): char;

  begin
    if character in ['a'..'z'] then
      UpCase := Chr(Ord(character) - Ord('a') + Ord('A'))
    else
      UpCase := character;
  end; {UpCase}

   {_________________________________________________}

  function DownCase (character: char): char;

  begin
    if character in ['A'..'Z'] then
      DownCase := Chr(Ord(character) - Ord('A') + Ord('a'))
    else
      DownCase := character;
  end; {DownCase}

   {_________________________________________________}

  function MenuChar (alpha, omega: char): char;

    var
      theChoice, alphaCap, omegaCap: char;
      inc: integer;
      goodChoice, capitals: boolean;

  begin
    capitals := false;
    if alpha in ['A'..'Z'] then
      begin
        capitals := true;
        alpha := DownCase(alpha);
        omega := DownCase(omega);
      end;

    goodChoice := false;
    inc := Ord('A') - Ord('a');
    alphaCap := Chr(Ord(alpha) + inc);
    omegaCap := Chr(Ord(omega) + inc);
    repeat
      if capitals then
        Writeln('    Choose a letter in the set [', alphaCap, '..', omegaCap, ']')
      else
        Writeln('    Choose a letter in the set [', alpha, '..', omega, ']');

      Readln(theChoice);
      if (theChoice in [alpha..omega]) or (theChoice in [alphaCap..omegaCap]) then
        goodChoice := true
      else
        SysBeep(3);
    until goodChoice;
    if theChoice in [alphaCap..omegaCap] then
      theChoice := Chr(Ord(theChoice) - inc);
    MenuChar := theChoice;
  end; {MenuChar}

   {_________________________________________________}

  procedure PageParameters (var SCREENBOX, POSTBOX: IBOX);

    var
      CTX, CTY: integer; {these parameters repesent the centre of the screen}
      boxWidth, boxHeight: real;

  begin {GraphParametersMod}
    Writeln('    Specify the frame surrounding the graph.');
    Writeln('    5cm x 3cm <= width x height <= 13.5cm x 9.5cm.');
    Writeln;
    Writeln('    For two diagrams per page use 9cm times 6cm');
    Writeln('    For three diagrams per page use 9cm times 3.75cm');
    Writeln;

    Writeln('              Specify the width ');
    ReadANumber(boxWidth);
    Writeln('              Specify the height ');
    ReadANumber(boxHeight);
    if boxHeight > 12.0 then
      boxHeight := 12.0;
    if boxHeight < 3 then
      boxHeight := 3;
    if boxWidth > 18.0 then
      boxWidth := 18.0;
    if boxWidth < 5 then
      boxWidth := 5;

    CTX := SCREENBOX.XC;{this is initally set to zero}
    CTY := SCREENBOX.YC; {this is initally set to zero}
    SCREENBOX.X0 := CTX;
    SCREENBOX.Y0 := CTY;
    SCREENBOX.X1 := CTX - Round((boxWidth * 36) / 2.54); {= HRS}
    SCREENBOX.Y1 := CTY - Round(boxHeight * 14.5); {= VTS}
    SCREENBOX.X2 := CTX + Round((boxWidth * 36) / 2.54); {= HRF}
    SCREENBOX.Y2 := CTY + Round(boxHeight * 14.5); {= VTF}

    POSTBOX.X1 := 0; {=XS}
    POSTBOX.Y1 := 0; {=YS}
    POSTBOX.X2 := 2 * Round((boxWidth * 36) / 2.54); {= XF}
    POSTBOX.Y2 := 2 * Round((boxHeight * 36) / 2.54); {= YF}

  end; {Page Parameters}

   {________________________________________________}

  procedure PageSetUp (var SCREENBOX, POSTBOX: IBOX;
                  var PSPict, TeXFormat: boolean);

  begin
    ClrScr;
    Writeln;
    Writeln('     SPECIFY THE PAGE PARAMETERS');
    Writeln;
    YesOrNo('     Do you want to output in Postcript?', PSPict);
    if PSPict then
      YesOrNo('     Do you want a TeXtures Format?', TeXFormat);

    PageParameters(SCREENBOX, POSTBOX);

    SCREENBOX.xAxis := true;
    SCREENBOX.yAxis := true;
    SCREENBOX.doFrame := true;
    SCREENBOX.doMarks := true;

    POSTBOX.xAxis := true;
    POSTBOX.yAxis := true;
    POSTBOX.doFrame := true;
    POSTBOX.doMarks := true;

    noPageParameters := false;
  end; {Page SetUp}

   {________________________________________________}

  procedure StripLeadingBlanks (var theString: string);

    const {ASCII Codes}
      TAB = 9;
      LINEFEED = 10;
      SPACE = 32;

    var
      exeat: boolean;

  begin
    exeat := false;
    repeat
      if Length(theString) <> 0 then
        begin
          if (Ord(theString[1]) in [SPACE, TAB, LINEFEED]) then
            Delete(theString, 1, 1)
          else
            exeat := true;
        end;
    until (Length(theString) = 0) or exeat;
  end; {StripLeadingBlanks}

   {_________________________________________________}

  procedure StripTrailingBlanks (var theString: string);

    const {ASCII Codes}
      TAB = 9;
      SPACE = 32;

    var
      n, m: integer;

  begin {StripTrailingBlanks}
    n := Length(theString);
    if n <> 0 then
      begin {if n<>0}
        repeat
          m := n;
          if (Ord(theString[n]) = SPACE) or (Ord(theString[n]) = TAB) then
            begin
              Delete(theString, n, 1);
              m := n - 1;
            end;
          n := n - 1;
        until (m <> n);
      end; {if n<>0}
  end; {StripTrailingBlanks}

   {_________________________________________________}

  procedure ReplaceEveryCaseOf (deletion, replacement: string;
                  var theString: string);

    var
      a, n: integer;
      newString, tempString: string;

  begin
    newString := '';
    n := Length(deletion);
    repeat
      a := Pos(deletion, theString);
      if a <> 0 then
        begin
          tempString := Copy(theString, 1, a - 1);
          tempString := Concat(tempString, replacement);
          Delete(theString, 1, a + n - 1);
          newString := Concat(newString, tempString);
        end;
    until a = 0;
    theString := Concat(newString, theString);

  end; {ReplaceEveryCaseOf}

   {_________________________________________________}

  procedure TallyPoints (range: real;
                  var taly: real);

    var
      PWR, TRNC: integer;
      FACT, LogBaseTen: real;

  begin {TallyPoints}
    logBaseTen := Ln(range) / Ln(10);
    if logBaseTen > 0 then
      begin
        PWR := Trunc(logBaseTen) - 1;
        FACT := Exp(PWR * Ln(10));
        TRNC := Round(range / FACT)
      end;
    if logBaseTen = 0 then
      begin
        FACT := 0.1;
        TRNC := 10
      end;
    if logBaseTen < 0 then
      begin
        PWR := Trunc(Abs(logBaseTen)) + 2;
        FACT := Exp(PWR * Ln(10));
        TRNC := Round(range * FACT);
        FACT := 1 / FACT;
      end;
    if (TRNC >= 10) and (TRNC < 15) then
      TALY := 2.5 * FACT;
    if (TRNC >= 15) and (TRNC < 30) then
      TALY := 5 * FACT;
    if (TRNC >= 30) and (TRNC < 60) then
      TALY := 10 * FACT;
    if (TRNC >= 60) and (TRNC <= 100) then
      TALY := 20 * FACT;

  end; {TallyPoints}

   {_________________________________________________}

  function IntegerToString (theInteger: integer): string;

    var
      i, n, digit: integer;
      theString, signChar: string;

  begin
    theString := '';
    signchar := '';
    if theInteger < 0 then
      begin
        signchar := '-';
        theInteger := -theInteger;
      end;

    if theInteger = 0 then
      theString := '0'
    else if (theInteger > 0) then
      begin {The Integer > 0}
        n := Trunc(Ln(theInteger) / Ln(10)) + 1;
        for i := 1 to n do
          begin {i}
            digit := theinteger mod 10;
            insert(Chr(digit + 48), thestring, 1);
            theinteger := theInteger div 10;
          end; {i}
      end; {The Integer > 0}
    theString := Concat(signChar, theString);
    IntegerToString := theString;
  end; {IntegerToString}

   {_________________________________________________}

  function TalyToString (theNumber: real): string;

    var
      scientific, decimal: boolean;
      sign, leadingDigits, digits, divisor: integer;
      i, p, q, r: integer;
      realNumber, log10: real;
      headString, powerString, numbString: string;

  begin
      {0. Determine the sign of the number and take the modulus}
    p := 0;
    sign := 1;
    if theNumber < 0 then
      begin
        theNumber := -theNumber;
        sign := -1;
      end;

      {1.Extract leading digits and determine the value of the exponent p}
    if theNumber <> 0 then
      begin {theNumber <>0}
        digits := 4;
        log10 := Ln(theNumber) / Ln(10.0);
        p := Trunc(log10);
        if theNumber < 1 then
          p := p - 1;
        realNumber := theNumber / Exp((p - digits) * Ln(10));
        leadingDigits := Round(realNumber / 10);
        if leadingDigits = 10000 then
          begin
            p := p + 1;
            leadingDigits := 1000;
          end;

      {2.Determine the number of nonzero digits}
        divisor := 10;
        q := 4;
        while leadingDigits mod divisor = 0 do
          begin
            divisor := divisor * 10;
            q := q - 1;
          end;
        r := q; {the original number of nonzero digits}
        if (p > 0) and (q < p + 1) then {not if p >4}
          q := Min(4, p + 1); {We need to count the integer zeros}
    {We need to count the integer zeros}

      {3. Determine whether or not to use Scientific notation}
        scientific := true;
        decimal := false;
        if (6 > (q - p)) and ((q - p) > 0) then
          begin
            scientific := false;
            decimal := true;
          end;
      end; {theNumber <>0}

      {4. Construct the Scientific  notation}
    if scientific and (theNumber <> 0) then
      begin {Scientific Notation}
        q := r;
        while ((leadingDigits mod 10) = 0) do
          leadingDigits := leadingDigits div 10;
        numbString := IntegerToString(leadingDigits);
        if q > 1 then
          Insert('.', numbString, 2)
        else if q = 1 then
          numbString := Concat(numbString, '.0');
        if p <> 0 then
          begin {Scientific Notation}
            powerString := IntegerToString(p);
            if p > 0 then
              powerString := Concat('+', powerString);
            numbString := Concat(numbString, 'e', powerString);
          end;{Scientific Notation}
      end; {Scientific Notation}

      {5. Construct the Decimal  notation}
    if Decimal and (theNumber <> 0) then
      begin
        for i := 1 to 4 - q do
          begin
            if (leadingDigits mod 10 = 0) then
              leadingDigits := leadingDigits div 10;
          end;
        numbString := IntegerToString(leadingDigits);
        ;
        if (p >= 0) and (p <= 2) and (q > p + 1) then
          Insert('.', numbString, p + 2);
        if p < 0 then
          begin {p <0}
            headString := '0.';
            for i := 2 to -p do
              headString := Concat(headString, '0');
            numbString := Concat(headString, numbString);
          end; {p <0}
      end; {Decimal}

    if sign = -1 then
      numbString := Concat('-', numbString);
    if theNumber = 0 then
      numbString := '0';

    TalyToString := numbString;

  end; {TalyToString}

   {_________________________________________________}

  function IntegerString (theString: string): boolean;

    var
      numeric: boolean;
      i, n: integer;

  begin {IntegerString}
    numeric := true;
    n := Length(theString);
    if n > 5 then
      numeric := false;
    if (not (theString[1] in ['0'..'9'])) and (not (theString[1] = '-')) then
      numeric := false;
    for i := 2 to n do
      begin {i}
        if (not (theString[i] in ['0'..'9'])) then
          begin
            numeric := false;
          end;
      end; {i}
    IntegerString := numeric;
  end; {IntegerString}

   {_________________________________________________}

  function DecimalString (theString: string): boolean;

    var
      numeric: boolean;
      i, n, points: integer;

  begin {DecimalString}
    points := 0;
    numeric := true;
    n := length(theString);
    if n <= 0 then
      numeric := false
    else
      begin {n>0}
        if not (theString[1] in ['0'..'9', '.', '-']) then
          numeric := false;
        if theString[1] = '.' then
          points := points + 1;
        for i := 2 to n do
          begin {i}
            if not (theString[i] in ['0'..'9', '.']) then
              numeric := false;
            if theString[i] = '.' then
              points := points + 1;
          end; {i}
      end; {n>0}
    if points > 1 then
      numeric := false;
    DecimalString := numeric;
  end; {DecimalString: Is it a Decimal String?}

   {_________________________________________________}

  function StringToInteger (theString: string): longint;

    var
      i, n, start: integer;
      thenumber: longint;
      negative: boolean;

  begin
    start := 1;
    negative := false;
    if theString[1] = '-' then
      begin
        negative := true;
        start := 2;
      end;
    n := Length(theString);
    theNumber := 0;
    for i := start to n do
      theNumber := theNumber * 10 + Ord(theString[i]) - Ord('0');
      {Usually Ord('0') = 48}
    StringToInteger := theNumber;
    if negative then
      StringToInteger := -theNumber;
  end; {StringToInteger}

   {_________________________________________________}

  function StringToDecimal (theString: string): real;

    var
      i, n, p, d: integer;
      theNumber, denominator: real;
      negative: boolean;

  begin{StringToDecimal}
    negative := false;
    if theString[1] = '-' then
      begin
        negative := true;
        ;
        delete(theString, 1, 1);
      end;

    n := length(theString);
    p := pos('.', theString);
    if p = 0 then
      p := n;
    d := n - p;

    theNumber := 0;
    if p <> n then
      begin
        delete(theString, p, 1);
        n := n - 1
      end;

    for i := 1 to n do
      theNumber := theNumber * 10 + Ord(thestring[i]) - Ord('0');
          {Usually ord('0') = 48}
    denominator := 1;

    for i := 1 to d do
      denominator := denominator * 10;

    StringToDecimal := theNumber / denominator;
    if negative then
      StringToDecimal := -theNumber / denominator
  end; {StringToDecimal}

   {_________________________________________________}

  procedure GetBoundedInteger (lowerBound, upperBound: integer;
                  var theInteger: integer);

    var
      theString: string;
      theLongint: longint;
      itsAnInteger, withinBounds: boolean;

  begin
      {Writeln('    Type an integer in the interval [', lowerBound : 1, ', ', upperBound : 3, '] ');}
    itsAnInteger := false;
    repeat {until itsAnInteger;}
      repeat
        Readln(theString);
      until theString <> '';
      itsAnInteger := IntegerString(theString);
      if itsAnInteger = false then
        begin {this is a rubbish string}
          Sysbeep(3);
          Writeln('    Type an integer in the interval [', lowerBound : 1, ', ', upperBound : 1, '] ');
          Writeln;
        end {this is a rubbish string}

      else {it is an integer String}

        begin {Convert the IntegerString}
          withinBounds := false;
          theLongint := StringToInteger(theString);
          if theLongint > upperBound then
            begin
              Sysbeep(3);
              Writeln('    This integer exceeds ', upperBound : 1, '! (Try again).');
            end
          else if theLongint < lowerBound then
            begin
              Sysbeep(3);
              Writeln('    This integer is less than ', lowerBound : 1, '! (Try again).');
            end
          else
            begin
              theInteger := theLongint;
              withinBounds := true;
            end;
        end; {Convert the IntegerString}
    until withinBounds;
  end; {GetBoundedInt}

   {_________________________________________________}

  procedure ReadAChar (var theCharacter: char);
    var
      charString: string;
  begin
    charString := '';
    repeat
      Readln(charString);
      StripLeadingBlanks(charString);
      StripTrailingBlanks(charString);
      if Length(CharString) <> 1 then
        begin
          Sysbeep(3);
          Writeln('    Type a single character!');
        end;
    until Length(CharString) = 1;
    theCharacter := charString[1];
  end; {ReadAChar}

   {_________________________________________________}

  procedure ReadANumber (var theNumber: real);

    var
      theString: string;
      itsANumber: boolean;

  begin
    itsANumber := false;
    repeat
      Readln(theString);
      StripLeadingBlanks(theString);
      StripTrailingBlanks(theString);
      itsANumber := DecimalString(theString);
      if itsANumber = false then
        begin
          Sysbeep(3);
          Write('Type a number ! ');
        end;
    until itsANumber;
    theNumber := StringToDecimal(theString)
  end; {ReadANumber}

   {_________________________________________________}

  procedure ReadBoundedNumber (lowerBound, upperBound: real;
                  var theNumber: real);

    var
      accepted: boolean;

  begin

    repeat
      ReadANumber(theNumber);

      accepted := false;
      if (theNumber < lowerBound) then
        begin
          Sysbeep(3);
          if lowerbound = 0.0 then
            Writeln('    The number should be positive!')
          else
            Writeln('    This number is less that the minimum value ', lowerBound : 5 : 2);
          Writeln('    Input another number');
        end
      else if (theNumber > upperBound) then
        begin
          Sysbeep(3);
          Writeln('    This number is greater that the maximum value ', upperBound : 5 : 2);
          Writeln('    Input another number');
        end
      else
        accepted := true;
    until accepted;

  end;{ReadBoundedNumber}

   {_________________________________________________}

  function LineStatus (theLine: string): integer;

    const
      TAB = 9;
      SPACE = 32;
      numericLine = 1;
      commaSeparated = 2;
      nonNumeric = 3;

    var
      i, n: integer;
      alphaChar, numericChar: boolean;
      blankChar, periodChar, commaChar, signChar, otherChar: boolean;

  begin
    alphaChar := false;
    numericChar := false;
    blankChar := false;
    commaChar := false;
    periodChar := false;
    signChar := false;
    otherChar := false;

    n := length(theLine);
    i := 1;
    repeat
      if theLine[i] in ['A'..'z'] then
        alphaChar := true
      else if theLine[i] in ['0'..'9'] then
        numericChar := true
      else if theLine[i] in ['+'..'-'] then
        signChar := true
      else if theLine[i] = '.' then
        periodChar := true
      else if Ord(theLine[i]) in [SPACE, TAB] then
        blankChar := true
      else if theLine[i] = ',' then
        commaChar := true
      else
        otherChar := true;
      i := i + 1;
    until alphaChar or (i = n + 1);
    if alphaChar or otherChar then
      LineStatus := nonNumeric
    else {if (alphaChar = false) and (otherChar = false) then}
      begin
        if numericChar and commaChar then
          LineStatus := commaSeparated
        else
          LineStatus := numericLine;
      end;
  end; {LineStatus}

   {_________________________________________________}

  procedure ReadANumericLine (theLine: string;
                  var foundNo: integer;
                  var numberVec: vector);

    const {ASCII Codes}
      TAB = 9;
      SPACE = 32;

    var
      i: integer;
      theNumber: real;
      numberString: string;
  begin
    i := 0;
    repeat
      ReplaceEveryCaseOf(',', ' ', theLine);
      StripLeadingBlanks(theLine);
      StripTrailingBlanks(theLine);
      numberString := '';
      repeat
        numberString := Concat(numberString, theLine[1]);
        Delete(theLine, 1, 1);
      until (Ord(theline[1]) in [SPACE, TAB]) or (Length(theLine) = 0);

      theNumber := StringToDecimal(numberString);
      i := i + 1;
      numberVec[i] := theNumber;
    until (Length(theLine) = 0) or (i = 50);
    foundNo := i;
  end; {ReadANumericLIne}

{_________________________________________________}

procedure ReadDataSafely(var realBOX: rBOX;
  var dataVector: longVector; maxArray: integer);

const
  numericLine = 1;
  commaSeparated = 2;
  nonNumeric = 3;

var
  dataSeries: pointsVector;
  quit, alphaAlert, goForIt: boolean;
  univariate, indexed, joinLines: boolean;
  monotonicity, linearity: boolean;
  i, j, statusOfLine, lineNumber: integer;
  foundNo: integer;
  yStore, startSlope, increment: real;
  numberVec: vector;
  dataFileName, theLine: string;
  Q: Text;

begin
  quit := False;
  goForIt := False;
  alphaAlert := False;
  {indexed := False;}
  {realBOX.noData := True;}
  realBOX.NOPUNCT := 0;
  i := -1;
  lineNumber := 0;
  yStore := 0.0;

  Write('      Name the new data file = ');
  Readln(dataFileName);
  if FileExists(dataFileName) = False then
  begin
    Writeln;
    Writeln('      There is no such file in the directory ');
    Writeln;
    Writeln(' ': 60, '<RETURN>');
    Readln;
    thereIsData := false;
    empiricData := false;
  end
  else if FileExists(dataFileName) = True then
  begin {dataFileName <> ''}
    realBOX.NOPUNCT := 0;
    Assign(Q, dataFileName);
    Reset(Q);
    while (not quit) and (i < maxArray) do
    begin {while not quit}
      Readln(Q, theLine);
      lineNumber := lineNumber + 1;
      statusOfLine := LineStatus(theLine);

      if statusOfLine = nonNumeric then
      begin {nonNumeric}
        if (alphaAlert = False) and (lineNumber <= 5) then
        begin {lineNumber <= 5}
          SysBeep(3);
          Writeln;
          Writeln('     ALERT!');
          Writeln;
          Writeln(
            '     This file  contains non-numeric symbols. Unless these  are confined');
          Writeln(
            '     to the top of the file, the attempt to read it will be discontinued');
          Writeln;
          Writeln(' ': 64, '<RETURN>');
          Readln;
          alphaAlert := True;
        end; {lineNumber <= 5}
        if (lineNumber > 5) then
        begin {lineNumber > 5}
          SysBeep(3);
          Writeln;
          Writeln('     ABORT!');
          Writeln;
          Writeln(
            '     This  file contains non-numeric symbols which are  not confined to');
          Writeln(
            '     the top of the file. The attempt to read it has been discontinued.');
          Writeln;
          if i > 0 then
            Writeln('     A total of ', i + 1: 1,
              ' data points have been read.');
          Writeln(' ': 64, '<RETURN>');
          Readln;
          quit := True;
          goForIt := False;
          foundNo := 0;
        end; {lineNumber > 5}
      end; {nonNumeric}

      if (statusOfLine = numericLine) or (statusOfLine = commaSeparated) then
      begin {Numeric or Comma Separated}

        ReadANumericLine(theLine, foundNo, numberVec);
        if (i + foundNo > maxArray) then
          quit := True;

        if (goForIt = False) then
        begin {Testing}
          univariate := False;
          indexed := False;
          joinLines := False;

          if foundNo = 1 then
          begin
            Writeln;
            Writeln(
              '     The line which  has just been read contains a single number. This');
            Writeln(
              '     will be regarded as an observation  on a univariate  series which');
            Writeln(
              '     will be  associated with  an index  sequence starting at zero and');
            Writeln('     increasing by one with each new observation.');
            Writeln;
            Writeln(' ': 64, '<RETURN>');
            Readln;
            univariate := True;
            goForIt := True;
          end;
          if foundNo = 2 then
          begin
            Writeln;
            Writeln(
              '     The line which  has been read contains two numbers. The requirement');
            Writeln(
              '     is for a univariate series  which may be accompanied by a serial or');
            Writeln(
              '     index  number which must preceed the observation. The first  number');
            Writeln(
              '     will be regarded as the index. It will be  checked for consistency.');
            Writeln;
            Writeln(' ': 64, '<RETURN>');
            Readln;
            indexed := True;
            goForIt := True;
          end;
          if not (foundNo in [1, 2]) then
          begin {foundNo in [1, 2]}
            SysBeep(3);
            Writeln;
            Writeln('     DECIDE!');
            Writeln;
            Writeln('     The line which has been read contains ',
              foundNo: 1, ' numbers. The requirement');
            Writeln(
              '     is for a univariate series  which may be accompanied by a serial');
            Writeln(
              '     or index  number which must preceed the observation. Do you wish');
            YesOrNo('     to construct a univariate series by joining the lines.', joinLines);
            if joinLines then
              goForIt := True
            else
            begin {ABORT}
              Writeln;
              SysBeep(3);
              Writeln;
              Writeln('     ABORT!');
              Writeln;
              Writeln('     The attempt to read the file has been discontinued .');
              Writeln;
              Writeln(' ': 64, '<RETURN>');
              quit := True;
              goForIt := False;
              Readln;
            end; {ABORT}
          end; {foundNo in [1, 2]}
        end; {Testing}
      end;{Numeric or Comma Separated}

      if goForIt then
      begin {goForit}

        if univariate then
        begin {univariate}
          i := i + 1;
          dataSeries[i].x := i;
          dataSeries[i].y := numberVec[1];
        end; {univariate}

        if indexed then
        begin {indexed}
          i := i + 1;
          dataSeries[i].x := numberVec[1];
          dataSeries[i].y := numberVec[2];
        end; {indexed}

        if (joinLines and not quit) then
        begin {joinLines: if i + foundNo < maxArray}
          for j := 1 to foundNo do
          begin {j}
            dataSeries[i + j].x := i + j;
            dataSeries[i + j].y := numberVec[j];
          end; {j}
          i := i + foundNo;
        end; {joinLines}

      end; {goForIt}

      if EOF(Q) then
        quit := True;
    end; {while not quit}

    realBOX.NOPUNCT := i + 1;
    Close(Q);

    monotonicity := True;
    linearity := True;
    startSlope := dataSeries[1].x - dataSeries[0].x;

    for i := 0 to realBOX.NOPUNCT - 1 do
    begin {i}
      if i = 0 then
      begin {i = 0}
        realBOX.yMax := dataSeries[0].y;
        realBOX.yMin := dataSeries[0].y;
        realBOX.xMax := dataSeries[0].x;
        realBOX.xMin := dataSeries[0].x;
      end {i = 0}
      else
      begin {i <> 0}
        increment := dataSeries[i].x - dataSeries[i - 1].x;
        if (increment < 0) then
          monotonicity := False;
        if (increment <> startSlope) then
          linearity := False;
        realBOX.xMax := Rmax(dataSeries[i].x, realBOX.xMax);
        realBOX.xMin := Rmin(dataSeries[i].x, realBOX.xMin);
        realBOX.yMax := Rmax(dataSeries[i].y, realBOX.yMax);
        realBOX.yMin := Rmin(dataSeries[i].y, realBOX.yMin);
      end; {i <> 0}
      yStore := yStore + dataSeries[i].y;
      dataVector[i] := dataSeries[i].y;
    end;{i}

    if indexed then
    begin {indexed series}
      if (linearity) then
      begin {linearity}
        Writeln('     The index which accompanies the series is linear');
        Writeln('     (i.e. it has a constant increment). ');
      end {linearity}
      else if monotonicity then
      begin {nonlinearity}
        Writeln('     The index which accompanies the series is monotonic');
        Writeln('     (i.e. it  increases  with each observation), but it');
        Writeln('     is nonlinear i.e. it  contains unequal increments).');
      end {nonlinearity}
      else
      begin {non-moniticity}
        Writeln('     The index which accompanies  the series is non-monotonic');
        Writeln('     (i.e. it both increases and decreases.) This will cause');
        Writeln('     problems in plotting the series.');
      end; {non-moniticity}
      Writeln(' ': 64, '<RETURN>');
      Readln;
    end; {indexed series}

    if realBOX.NOPUNCT > 0 then
    begin {realBOX.NOPUNCT > 0}
      realBOX.xLB := realBOX.xMin;
      realBOX.xRB := realBOX.xMax;
      realBOX.yMean := yStore / realBOX.NOPUNCT;

      realBOX.yVar := 0;
      for i := 0 to realBOX.NOPUNCT - 1 do
      begin {i}
        realBOX.yVar := realBOX.yVar + Sqr(dataVector[i] - realBOX.yMean);
      end; {i}
      realBOX.yVar := realBOX.yVar / realBOX.NOPUNCT;
      {realBOX.noData := False;}
      thereIsData := true;
    end; {realBOX.NOPUNCT > 0}
  end;{dataFileName <> ''}

end; {ReadDataSafely}

{_________________________________________________}

  procedure ReadTheData (var dataBOX: rBOX;
                  var dataVector: longVector);
    var
      decision: boolean;

  begin
    ClrScr;
    Writeln;
    Writeln;
    Writeln('    READ A DATA FILE');
    Writeln;
    ReadDataSafely(dataBOX, dataVector, maxArray);

    if thereIsData then
    begin {Name the data}
      empiricData := true;
      dataName := 'an unidentified data series';

      ClrScr;
      Writeln;
      Writeln;
      Writeln('    NAME THE DATA');
      Writeln;
      Writeln('    By default the data will described as');
      Writeln;
      Writeln('    <<an unidentified data series>>');
      Writeln;
      YesorNo('    Do you wish to rename the data', decision);
      if decision = true then
        begin
          Writeln('    Name the data series without refering to its frequency');
          Write('    The name of the data = ');
          Readln(dataName);
        end
      else
        dataName := 'an unidentified data series';
    end; {Name the data}

    ARMAModel := false;
    {empiricData := true;}
    pseudoData := false;
    ARestimates := false;
    ARMAGen.bandLimits := false;
    ARMAGen.contaminated := false;

  end; {ReadTheData}

   {_________________________________________________}

  procedure WriteData (var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      i, Tcap: integer;
      serials: boolean;
      dataFileName: string;
      F: text;

  begin {WriteData}
    Tcap := dataBOX.NOPUNCT;
    YesOrNo('    Do you want to have serial numbers', serials);
    Write('    Name the new data file = ');
    Readln(dataFileName);
    if dataFileName <> '' then
      begin {if}
        Assign(F, dataFileName);
        Rewrite(F);
        Writeln(F);
        for i := 0 to Tcap - 1 do
          begin {i}
            if serials then
              Writeln(F, i : 3, '  ', dataVector[i] : 10 : 5)
            else
              Writeln(F, '  ', dataVector[i] : 10 : 5)
          end; {i}
        Close(F);
      end; {if}
  end; {WriteData}

   {_________________________________________________}

  procedure MakeDataBox (var dataBOX: rBOX;
                  var dataVector: longVector);

 {We need only supply this procdure with the dataVector and the value of dataVector.NOPUNCT}
    var
      dataMean, minValue, maxValue, yRange: real;
      t: integer;

  begin
    dataMean := 0.0;
    for t := 0 to dataBOX.NOPUNCT - 1 do
      begin {t}
        dataMean := dataMean + dataVector[t];
        if t = 0 then
          begin {if}
            minValue := dataVector[0];
            maxValue := dataVector[0];
          end {if}
        else
          begin {else}
            maxValue := Rmax(maxValue, dataVector[t]);
            minValue := Rmin(minValue, dataVector[t]);
          end; {else}
      end; {t}
    dataMean := dataMean / dataBOX.NOPUNCT;

    dataBOX.xMin := 0.0;
    dataBOX.xMax := dataBOX.NOPUNCT - 1;
    dataBOX.xLB := 0.0;
    dataBOX.xRB := dataBOX.NOPUNCT - 1;
    dataBOX.yMin := minValue;
    dataBOX.yMax := maxValue;
    dataBOX.yMean := dataMean;
    dataBOX.yUB := dataBOX.yMax;
    dataBOX.yLB := dataBOX.yMin;

    yRange := dataBOX.yMax - dataBOX.yMin;
    dataBOX.yUB := dataBOX.yUB + 0.2 * yRange;
    dataBOX.yLB := dataBOX.yLB - 0.2 * yRange;

  end; {MakeDataBox}

   {________________________________________________}

end. {UTILS: Unit 3 of  BLIMDOS.PAS}

